@@ -27,12 +27,22 @@ class NearestNDInterpolator(NDInterpolatorBase):
 
     .. versionadded:: 0.9
 
+    Methods
+    -------
+    __call__
+
     Parameters
     ----------
     points : (Npoints, Ndims) ndarray of floats
         Data point coordinates.
     values : (Npoints,) ndarray of float or complex
         Data values.
+    rescale : boolean, optional
+        Rescale points to unit cube before performing interpolation.
+        This is useful if some of the input dimensions have
+        incommensurable units and differ by many orders of magnitude.
+
+        .. versionadded:: 0.14.0
 
     Notes
     -----
@@ -40,11 +50,11 @@ class NearestNDInterpolator(NDInterpolatorBase):
 
     """
 
-    def __init__(self, x, y):
-        x = _ndim_coords_from_arrays(x)
-        self._check_init_shape(x, y)
-        self.tree = cKDTree(x)
-        self.points = x
+    def __init__(self, x, y, rescale=False):
+        NDInterpolatorBase.__init__(self, x, y, rescale=rescale,
+                                    need_contiguous=False,
+                                    need_values=False)
+        self.tree = cKDTree(self.points)
         self.values = y
 
     def __call__(self, *args):
@@ -57,8 +67,9 @@ class NearestNDInterpolator(NDInterpolatorBase):
             Points where to interpolate data at.
 
         """
-        xi = _ndim_coords_from_arrays(args)
+        xi = _ndim_coords_from_arrays(args, ndim=self.points.shape[1])
         xi = self._check_call_shape(xi)
+        xi = self._scale_x(xi)
         dist, i = self.tree.query(xi)
         return self.values[i]
 
@@ -67,22 +78,20 @@ class NearestNDInterpolator(NDInterpolatorBase):
 # Convenience interface function
 #------------------------------------------------------------------------------
 
-def griddata(points, values, xi, method='linear', fill_value=np.nan):
+def griddata(points, values, xi, method='linear', fill_value=np.nan,
+             rescale=False):
     """
-    Interpolate unstructured N-dimensional data.
-
-    .. versionadded:: 0.9
+    Interpolate unstructured D-dimensional data.
 
     Parameters
     ----------
-    points : ndarray of floats, shape (N, ndim)
+    points : ndarray of floats, shape (n, D)
         Data point coordinates. Can either be an array of
-        size (N, ndim), or a tuple of `ndim` arrays.
-    values : ndarray of float or complex, shape (N,)
+        shape (n, D), or a tuple of `ndim` arrays.
+    values : ndarray of float or complex, shape (n,)
         Data values.
-    xi : ndarray of float, shape (M, ndim)
+    xi : ndarray of float, shape (M, D)
         Points at which to interpolate data.
-
     method : {'linear', 'nearest', 'cubic'}, optional
         Method of interpolation. One of
 
@@ -105,13 +114,22 @@ def griddata(points, values, xi, method='linear', fill_value=np.nan):
           piecewise cubic, continuously differentiable (C1), and
           approximately curvature-minimizing polynomial surface. See
           `CloughTocher2DInterpolator` for more details.
-
     fill_value : float, optional
         Value used to fill in for requested points outside of the
         convex hull of the input points.  If not provided, then the
         default is ``nan``. This option has no effect for the
         'nearest' method.
+    rescale : boolean, optional
+        Rescale points to unit cube before performing interpolation.
+        This is useful if some of the input dimensions have
+        incommensurable units and differ by many orders of magnitude.
 
+        .. versionadded:: 0.14.0
+
+    Notes
+    -----
+
+    .. versionadded:: 0.9
 
     Examples
     --------
@@ -183,13 +201,15 @@ def griddata(points, values, xi, method='linear', fill_value=np.nan):
                       fill_value=fill_value)
         return ip(xi)
     elif method == 'nearest':
-        ip = NearestNDInterpolator(points, values)
+        ip = NearestNDInterpolator(points, values, rescale=rescale)
         return ip(xi)
     elif method == 'linear':
-        ip = LinearNDInterpolator(points, values, fill_value=fill_value)
+        ip = LinearNDInterpolator(points, values, fill_value=fill_value,
+                                  rescale=rescale)
         return ip(xi)
     elif method == 'cubic' and ndim == 2:
-        ip = CloughTocher2DInterpolator(points, values, fill_value=fill_value)
+        ip = CloughTocher2DInterpolator(points, values, fill_value=fill_value,
+                                        rescale=rescale)
         return ip(xi)
     else:
         raise ValueError("Unknown interpolation method %r for "
