@@ -1,81 +1,34 @@
-"""Module for reading and writing MATLAB .mat files"""
-# Authors: Travis Oliphant, Matthew Brett
-
 """
 Module for reading and writing matlab (TM) .mat files
 """
+# Authors: Travis Oliphant, Matthew Brett
 
-import os
-import sys
-import warnings
+from __future__ import division, print_function, absolute_import
 
-from numpy.compat import asbytes
+import numpy as np
 
-from miobase import get_matfile_version, docfiller
-from mio4 import MatFile4Reader, MatFile4Writer
-from mio5 import MatFile5Reader, MatFile5Writer
+from scipy.lib.six import string_types
 
-__all__ = ['find_mat_file', 'mat_reader_factory', 'loadmat', 'savemat']
+from .miobase import get_matfile_version, docfiller
+from .mio4 import MatFile4Reader, MatFile4Writer
+from .mio5 import MatFile5Reader, MatFile5Writer
 
-@docfiller
-def find_mat_file(file_name, appendmat=True):
-    ''' Try to find .mat file on system path
-
-    Parameters
-    ----------
-    file_name : str
-       file name for mat file
-    %(append_arg)s
-
-    Returns
-    -------
-    full_name : string
-       possibly modified name after path search
-    '''
-    warnings.warn('Searching for mat files on python system path will be ' +
-                  'removed in next version of scipy',
-                   DeprecationWarning, stacklevel=2)
-    if appendmat and file_name.endswith(".mat"):
-        file_name = file_name[:-4]
-    if os.sep in file_name:
-        full_name = file_name
-        if appendmat:
-            full_name = file_name + ".mat"
-    else:
-        full_name = None
-        junk, file_name = os.path.split(file_name)
-        for path in [os.curdir] + list(sys.path):
-            test_name = os.path.join(path, file_name)
-            if appendmat:
-                test_name += ".mat"
-            try:
-                fid = open(test_name,'rb')
-                fid.close()
-                full_name = test_name
-                break
-            except IOError:
-                pass
-    return full_name
+__all__ = ['mat_reader_factory', 'loadmat', 'savemat', 'whosmat']
 
 
 def _open_file(file_like, appendmat):
     ''' Open `file_like` and return as file-like object '''
-    if isinstance(file_like, basestring):
+    if isinstance(file_like, string_types):
         try:
             return open(file_like, 'rb')
-        except IOError:
-            pass
-        if appendmat and not file_like.endswith('.mat'):
-            try:
-                return open(file_like + '.mat', 'rb')
-            except IOError:
-                pass
-        # search the python path - we'll remove this soon
-        full_name = find_mat_file(file_like, appendmat)
-        if full_name is None:
-            raise IOError("%s not found on the path."
-                          % file_like)
-        return open(full_name, 'rb')
+        except IOError as e:
+            if appendmat and not file_like.endswith('.mat'):
+                file_like += '.mat'
+                try:
+                    return open(file_like, 'rb')
+                except IOError:
+                    pass  # Rethrow the original exception.
+            raise
     # not a string - maybe file-like object
     try:
         file_like.read(0)
@@ -112,19 +65,49 @@ def mat_reader_factory(file_name, appendmat=True, **kwargs):
     else:
         raise TypeError('Did not recognize version %s' % mjv)
 
+
 @docfiller
-def loadmat(file_name,  mdict=None, appendmat=True, **kwargs):
+def loadmat(file_name, mdict=None, appendmat=True, **kwargs):
     """
     Load MATLAB file
 
     Parameters
     ----------
-    %(file_arg)s
+    file_name : str
+       Name of the mat file (do not need .mat extension if
+       appendmat==True) Can also pass open file-like object.
     m_dict : dict, optional
         Dictionary in which to insert matfile variables.
-    %(append_arg)s
-    %(load_args)s
-    %(struct_arg)s
+    appendmat : bool, optional
+       True to append the .mat extension to the end of the given
+       filename, if not already present.
+    byte_order : str or None, optional
+       None by default, implying byte order guessed from mat
+       file. Otherwise can be one of ('native', '=', 'little', '<',
+       'BIG', '>').
+    mat_dtype : bool, optional
+       If True, return arrays in same dtype as would be loaded into
+       MATLAB (instead of the dtype with which they are saved).
+    squeeze_me : bool, optional
+       Whether to squeeze unit matrix dimensions or not.
+    chars_as_strings : bool, optional
+       Whether to convert char arrays to string arrays.
+    matlab_compatible : bool, optional
+       Returns matrices as would be loaded by MATLAB (implies
+       squeeze_me=False, chars_as_strings=False, mat_dtype=True,
+       struct_as_record=True).
+    struct_as_record : bool, optional
+       Whether to load MATLAB structs as numpy record arrays, or as
+       old-style numpy arrays with dtype=object.  Setting this flag to
+       False replicates the behavior of scipy version 0.7.x (returning
+       numpy object arrays).  The default setting is True, because it
+       allows easier round-trip load and save of MATLAB files.
+    verify_compressed_data_integrity : bool, optional
+        Whether the length of compressed sequences in the MATLAB file
+        should be checked, to ensure that they are not longer than we expect.
+        It is advisable to enable this (the default) because overlong
+        compressed sequences in MATLAB files generally indicate that the
+        files have experienced some sort of corruption.
     variable_names : None or sequence
         If None (the default) - read all variables in file. Otherwise
         `variable_names` should be a sequence of strings, giving names of the
@@ -154,17 +137,18 @@ def loadmat(file_name,  mdict=None, appendmat=True, **kwargs):
         mdict.update(matfile_dict)
     else:
         mdict = matfile_dict
-    if isinstance(file_name, basestring):
+    if isinstance(file_name, string_types):
         MR.mat_stream.close()
     return mdict
 
+
 @docfiller
 def savemat(file_name, mdict,
             appendmat=True,
             format='5',
             long_field_names=False,
             do_compression=False,
-            oned_as=None):
+            oned_as='row'):
     """
     Save a dictionary of names and arrays into a MATLAB-style .mat file.
 
@@ -177,49 +161,40 @@ def savemat(file_name, mdict,
         Name of the .mat file (.mat extension not needed if ``appendmat ==
         True``).
         Can also pass open file_like object.
-    m_dict : dict
+    mdict : dict
         Dictionary from which to save matfile variables.
-    %(append_arg)s
+    appendmat : bool, optional
+        True (the default) to append the .mat extension to the end of the
+        given filename, if not already present.
     format : {'5', '4'}, string, optional
         '5' (the default) for MATLAB 5 and up (to 7.2),
         '4' for MATLAB 4 .mat files
-    %(long_fields)s
-    %(do_compression)s
-    %(oned_as)s
+    long_field_names : bool, optional
+        False (the default) - maximum field name length in a structure is
+        31 characters which is the documented maximum length.
+        True - maximum field name length in a structure is 63 characters
+        which works for MATLAB 7.6+
+    do_compression : bool, optional
+        Whether or not to compress matrices on write.  Default is False.
+    oned_as : {'row', 'column'}, optional
+        If 'column', write 1-D numpy arrays as column vectors.
+        If 'row', write 1-D numpy arrays as row vectors.
 
     See also
     --------
     mio4.MatFile4Writer
     mio5.MatFile5Writer
-
-    Notes
-    -----
-    If ``format == '4'``, `mio4.MatFile4Writer` is called, which sets
-    `oned_as` to 'row' if it had been None.  If ``format == '5'``,
-    `mio5.MatFile5Writer` is called, which sets `oned_as` to 'column' if
-    it had been None, but first it executes:
-
-    ``warnings.warn("Using oned_as default value ('column')" +``
-                  ``" This will change to 'row' in future versions",``
-                  ``FutureWarning, stacklevel=2)``
-
-    without being more specific as to precisely when the change will take
-    place.
-
     """
-    file_is_string = isinstance(file_name, basestring)
+    file_is_string = isinstance(file_name, string_types)
     if file_is_string:
         if appendmat and file_name[-4:] != ".mat":
             file_name = file_name + ".mat"
         file_stream = open(file_name, 'wb')
     else:
-        try:
-            file_name.write(asbytes(''))
-        except AttributeError:
+        if not hasattr(file_name, 'write'):
             raise IOError('Writer needs file name or writeable '
                            'file-like object')
         file_stream = file_name
-
     if format == '4':
         if long_field_names:
             raise ValueError("Long field names are not available for version 4 files")
@@ -235,3 +210,42 @@ def savemat(file_name, mdict,
     MW.put_variables(mdict)
     if file_is_string:
         file_stream.close()
+
+
+@docfiller
+def whosmat(file_name, appendmat=True, **kwargs):
+    """
+    List variables inside a MATLAB file
+
+    Parameters
+    ----------
+    %(file_arg)s
+    %(append_arg)s
+    %(load_args)s
+    %(struct_arg)s
+
+    Returns
+    -------
+    variables : list of tuples
+        A list of tuples, where each tuple holds the matrix name (a string),
+        its shape (tuple of ints), and its data class (a string).
+        Possible data classes are: int8, uint8, int16, uint16, int32, uint32,
+        int64, uint64, single, double, cell, struct, object, char, sparse,
+        function, opaque, logical, unknown.
+
+    Notes
+    -----
+    v4 (Level 1.0), v6 and v7 to 7.2 matfiles are supported.
+
+    You will need an HDF5 python library to read matlab 7.3 format mat
+    files.  Because scipy does not supply one, we do not implement the
+    HDF5 / 7.3 interface here.
+
+    .. versionadded:: 0.12.0
+
+    """
+    ML = mat_reader_factory(file_name, **kwargs)
+    variables = ML.list_variables()
+    if isinstance(file_name, string_types):
+        ML.mat_stream.close()
+    return variables
