@@ -12,22 +12,24 @@
 # moved to their own files.  Still in this file are functions for eigenstuff
 # and for the Hessenberg form.
 
+from __future__ import division, print_function, absolute_import
+
 __all__ = ['eig','eigh','eig_banded','eigvals','eigvalsh', 'eigvals_banded',
            'hessenberg']
 
 import numpy
 from numpy import array, asarray_chkfinite, asarray, diag, zeros, ones, \
         isfinite, inexact, nonzero, iscomplexobj, cast, flatnonzero, conj
-
 # Local imports
-from scipy.linalg import calc_lwork
-from misc import LinAlgError, _datanotshared
-from lapack import get_lapack_funcs
-from blas import get_blas_funcs
+from scipy.lib.six import xrange
+from .misc import LinAlgError, _datacopied, norm
+from .lapack import get_lapack_funcs
+from .blas import get_blas_funcs
 
 
 _I = cast['F'](1j)
 
+
 def _make_complex_eigvecs(w, vin, dtype):
     """
     Produce complex-valued eigenvectors from LAPACK DGGEV real-valued output
@@ -35,24 +37,19 @@ def _make_complex_eigvecs(w, vin, dtype):
     # - see LAPACK man page DGGEV at ALPHAI
     v = numpy.array(vin, dtype=dtype)
     m = (w.imag > 0)
-    m[:-1] |= (w.imag[1:] < 0) # workaround for LAPACK bug, cf. ticket #709
+    m[:-1] |= (w.imag[1:] < 0)  # workaround for LAPACK bug, cf. ticket #709
     for i in flatnonzero(m):
         v.imag[:,i] = vin[:,i+1]
         conj(v[:,i], v[:,i+1])
     return v
 
-def _geneig(a1, b, left, right, overwrite_a, overwrite_b):
-    b1 = asarray(b)
-    overwrite_b = overwrite_b or _datanotshared(b1, b)
-    if len(b1.shape) != 2 or b1.shape[0] != b1.shape[1]:
-        raise ValueError('expected square matrix')
+
+def _geneig(a1, b1, left, right, overwrite_a, overwrite_b):
     ggev, = get_lapack_funcs(('ggev',), (a1, b1))
     cvl, cvr = left, right
-    if ggev.module_name[:7] == 'clapack':
-        raise NotImplementedError('calling ggev from %s' % ggev.module_name)
     res = ggev(a1, b1, lwork=-1)
-    lwork = res[-2][0]
-    if ggev.prefix in 'cz':
+    lwork = res[-2][0].real.astype(numpy.int)
+    if ggev.typecode in 'cz':
         alpha, beta, vl, vr, work, info = ggev(a1, b1, cvl, cvr, lwork,
                                                     overwrite_a, overwrite_b)
         w = alpha / beta
@@ -68,12 +65,20 @@ def _geneig(a1, b, left, right, overwrite_a, overwrite_b):
                                                                     % info)
 
     only_real = numpy.logical_and.reduce(numpy.equal(w.imag, 0.0))
-    if not (ggev.prefix in 'cz' or only_real):
+    if not (ggev.typecode in 'cz' or only_real):
         t = w.dtype.char
         if left:
             vl = _make_complex_eigvecs(w, vl, t)
         if right:
             vr = _make_complex_eigvecs(w, vr, t)
+
+    # the eigenvectors returned by the lapack function are NOT normalized
+    for i in xrange(vr.shape[0]):
+        if right:
+            vr[:, i] /= norm(vr[:, i])
+        if left:
+            vl[:, i] /= norm(vl[:, i])
+
     if not (left or right):
         return w
     if left:
@@ -82,95 +87,103 @@ def _geneig(a1, b, left, right, overwrite_a, overwrite_b):
         return w, vl
     return w, vr
 
-def eig(a, b=None, left=False, right=True, overwrite_a=False, overwrite_b=False):
-    """Solve an ordinary or generalized eigenvalue problem of a square matrix.
+
+def eig(a, b=None, left=False, right=True, overwrite_a=False,
+        overwrite_b=False, check_finite=True):
+    """
+    Solve an ordinary or generalized eigenvalue problem of a square matrix.
 
     Find eigenvalues w and right or left eigenvectors of a general matrix::
 
         a   vr[:,i] = w[i]        b   vr[:,i]
         a.H vl[:,i] = w[i].conj() b.H vl[:,i]
 
-    where .H is the Hermitean conjugation.
+    where ``.H`` is the Hermitian conjugation.
 
     Parameters
     ----------
-    a : array, shape (M, M)
+    a : (M, M) array_like
         A complex or real matrix whose eigenvalues and eigenvectors
         will be computed.
-    b : array, shape (M, M)
+    b : (M, M) array_like, optional
         Right-hand side matrix in a generalized eigenvalue problem.
-        If omitted, identity matrix is assumed.
-    left : boolean
-        Whether to calculate and return left eigenvectors
-    right : boolean
-        Whether to calculate and return right eigenvectors
-
-    overwrite_a : boolean
-        Whether to overwrite data in a (may improve performance)
-    overwrite_b : boolean
-        Whether to overwrite data in b (may improve performance)
+        Default is None, identity matrix is assumed.
+    left : bool, optional
+        Whether to calculate and return left eigenvectors.  Default is False.
+    right : bool, optional
+        Whether to calculate and return right eigenvectors.  Default is True.
+    overwrite_a : bool, optional
+        Whether to overwrite `a`; may improve performance.  Default is False.
+    overwrite_b : bool, optional
+        Whether to overwrite `b`; may improve performance.  Default is False.
+    check_finite : boolean, optional
+        Whether to check that the input matrices contain only finite numbers.
+        Disabling may give a performance gain, but may result in problems
+        (crashes, non-termination) if the inputs do contain infinities or NaNs.
 
     Returns
     -------
-    w : double or complex array, shape (M,)
+    w : (M,) double or complex ndarray
         The eigenvalues, each repeated according to its multiplicity.
+    vl : (M, M) double or complex ndarray
+        The normalized left eigenvector corresponding to the eigenvalue
+        ``w[i]`` is the column vl[:,i]. Only returned if ``left=True``.
+    vr : (M, M) double or complex ndarray
+        The normalized right eigenvector corresponding to the eigenvalue
+        ``w[i]`` is the column ``vr[:,i]``.  Only returned if ``right=True``.
 
-    (if left == True)
-    vl : double or complex array, shape (M, M)
-        The normalized left eigenvector corresponding to the eigenvalue w[i]
-        is the column v[:,i].
-
-    (if right == True)
-    vr : double or complex array, shape (M, M)
-        The normalized right eigenvector corresponding to the eigenvalue w[i]
-        is the column vr[:,i].
-
-    Raises LinAlgError if eigenvalue computation does not converge
+    Raises
+    ------
+    LinAlgError
+        If eigenvalue computation does not converge.
 
     See Also
     --------
-    eigh : eigenvalues and right eigenvectors for symmetric/Hermitian arrays
+    eigh : Eigenvalues and right eigenvectors for symmetric/Hermitian arrays.
 
     """
-    a1 = asarray_chkfinite(a)
+    if check_finite:
+        a1 = asarray_chkfinite(a)
+    else:
+        a1 = asarray(a)
     if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
         raise ValueError('expected square matrix')
-    overwrite_a = overwrite_a or (_datanotshared(a1, a))
+    overwrite_a = overwrite_a or (_datacopied(a1, a))
     if b is not None:
-        b = asarray_chkfinite(b)
-        if b.shape != a1.shape:
-            raise ValueError('a and b must have the same shape')
-        return _geneig(a1, b, left, right, overwrite_a, overwrite_b)
-    geev, = get_lapack_funcs(('geev',), (a1,))
-    compute_vl, compute_vr = left, right
-    if geev.module_name[:7] == 'flapack':
-        lwork = calc_lwork.geev(geev.prefix, a1.shape[0],
-                                    compute_vl, compute_vr)[1]
-        if geev.prefix in 'cz':
-            w, vl, vr, info = geev(a1, lwork=lwork,
-                                        compute_vl=compute_vl,
-                                        compute_vr=compute_vr,
-                                        overwrite_a=overwrite_a)
+        if check_finite:
+            b1 = asarray_chkfinite(b)
         else:
-            wr, wi, vl, vr, info = geev(a1, lwork=lwork,
-                                        compute_vl=compute_vl,
-                                        compute_vr=compute_vr,
-                                        overwrite_a=overwrite_a)
-            t = {'f':'F','d':'D'}[wr.dtype.char]
-            w = wr + _I * wi
-    else: # 'clapack'
-        if geev.prefix in 'cz':
-            w, vl, vr, info = geev(a1,
+            b1 = asarray(b)
+        overwrite_b = overwrite_b or _datacopied(b1, b)
+        if len(b1.shape) != 2 or b1.shape[0] != b1.shape[1]:
+            raise ValueError('expected square matrix')
+        if b1.shape != a1.shape:
+            raise ValueError('a and b must have the same shape')
+        return _geneig(a1, b1, left, right, overwrite_a, overwrite_b)
+
+    geev, geev_lwork = get_lapack_funcs(('geev', 'geev_lwork'), (a1,))
+    compute_vl, compute_vr = left, right
+
+    lwork, info = geev_lwork(a1.shape[0],
+                             compute_vl=compute_vl,
+                             compute_vr=compute_vr)
+    if info != 0:
+        raise LinAlgError("internal *geev work array calculation failed: %d" % (info,))
+    lwork = int(lwork.real)
+
+    if geev.typecode in 'cz':
+        w, vl, vr, info = geev(a1, lwork=lwork,
+                               compute_vl=compute_vl,
+                               compute_vr=compute_vr,
+                               overwrite_a=overwrite_a)
+    else:
+        wr, wi, vl, vr, info = geev(a1, lwork=lwork,
                                     compute_vl=compute_vl,
                                     compute_vr=compute_vr,
                                     overwrite_a=overwrite_a)
-        else:
-            wr, wi, vl, vr, info = geev(a1,
-                                        compute_vl=compute_vl,
-                                        compute_vr=compute_vr,
-                                        overwrite_a=overwrite_a)
-            t = {'f':'F','d':'D'}[wr.dtype.char]
-            w = wr + _I * wi
+        t = {'f':'F','d':'D'}[wr.dtype.char]
+        w = wr + _I * wi
+
     if info < 0:
         raise ValueError('illegal value in %d-th argument of internal geev'
                                                                     % -info)
@@ -179,7 +192,7 @@ def eig(a, b=None, left=False, right=True, overwrite_a=False, overwrite_b=False)
                             "with order >= %d have converged)" % info)
 
     only_real = numpy.logical_and.reduce(numpy.equal(w.imag, 0.0))
-    if not (geev.prefix in 'cz' or only_real):
+    if not (geev.typecode in 'cz' or only_real):
         t = w.dtype.char
         if left:
             vl = _make_complex_eigvecs(w, vl, t)
@@ -193,86 +206,110 @@ def eig(a, b=None, left=False, right=True, overwrite_a=False, overwrite_b=False)
         return w, vl
     return w, vr
 
+
 def eigh(a, b=None, lower=True, eigvals_only=False, overwrite_a=False,
-         overwrite_b=False, turbo=True, eigvals=None, type=1):
-    """Solve an ordinary or generalized eigenvalue problem for a complex
+         overwrite_b=False, turbo=True, eigvals=None, type=1,
+         check_finite=True):
+    """
+    Solve an ordinary or generalized eigenvalue problem for a complex
     Hermitian or real symmetric matrix.
 
-    Find eigenvalues w and optionally eigenvectors v of matrix a, where
-    b is positive definite::
+    Find eigenvalues w and optionally eigenvectors v of matrix `a`, where
+    `b` is positive definite::
 
                       a v[:,i] = w[i] b v[:,i]
         v[i,:].conj() a v[:,i] = w[i]
         v[i,:].conj() b v[:,i] = 1
 
-
     Parameters
     ----------
-    a : array, shape (M, M)
+    a : (M, M) array_like
         A complex Hermitian or real symmetric matrix whose eigenvalues and
         eigenvectors will be computed.
-    b : array, shape (M, M)
+    b : (M, M) array_like, optional
         A complex Hermitian or real symmetric definite positive matrix in.
         If omitted, identity matrix is assumed.
-    lower : boolean
+    lower : bool, optional
         Whether the pertinent array data is taken from the lower or upper
-        triangle of a. (Default: lower)
-    eigvals_only : boolean
+        triangle of `a`. (Default: lower)
+    eigvals_only : bool, optional
         Whether to calculate only eigenvalues and no eigenvectors.
         (Default: both are calculated)
-    turbo : boolean
+    turbo : bool, optional
         Use divide and conquer algorithm (faster but expensive in memory,
         only for generalized eigenvalue problem and if eigvals=None)
-    eigvals : tuple (lo, hi)
+    eigvals : tuple (lo, hi), optional
         Indexes of the smallest and largest (in ascending order) eigenvalues
-        and corresponding eigenvectors to be returned: 0 <= lo < hi <= M-1.
+        and corresponding eigenvectors to be returned: 0 <= lo <= hi <= M-1.
         If omitted, all eigenvalues and eigenvectors are returned.
-    type: integer
+    type : int, optional
         Specifies the problem type to be solved:
+
            type = 1: a   v[:,i] = w[i] b v[:,i]
+
            type = 2: a b v[:,i] = w[i]   v[:,i]
+
            type = 3: b a v[:,i] = w[i]   v[:,i]
-    overwrite_a : boolean
-        Whether to overwrite data in a (may improve performance)
-    overwrite_b : boolean
-        Whether to overwrite data in b (may improve performance)
+    overwrite_a : bool, optional
+        Whether to overwrite data in `a` (may improve performance)
+    overwrite_b : bool, optional
+        Whether to overwrite data in `b` (may improve performance)
+    check_finite : boolean, optional
+        Whether to check that the input matrices contain only finite numbers.
+        Disabling may give a performance gain, but may result in problems
+        (crashes, non-termination) if the inputs do contain infinities or NaNs.
 
     Returns
     -------
-    w : real array, shape (N,)
+    w : (N,) float ndarray
         The N (1<=N<=M) selected eigenvalues, in ascending order, each
         repeated according to its multiplicity.
+    v : (M, N) complex ndarray
+        (if eigvals_only == False)
 
-    (if eigvals_only == False)
-    v : complex array, shape (M, N)
         The normalized selected eigenvector corresponding to the
-        eigenvalue w[i] is the column v[:,i]. Normalization:
-        type 1 and 3:       v.conj() a      v  = w
-        type 2:        inv(v).conj() a  inv(v) = w
-        type = 1 or 2:      v.conj() b      v  = I
-        type = 3     :      v.conj() inv(b) v  = I
+        eigenvalue w[i] is the column v[:,i].
 
-    Raises LinAlgError if eigenvalue computation does not converge,
-    an error occurred, or b matrix is not definite positive. Note that
-    if input matrices are not symmetric or hermitian, no error is reported
-    but results will be wrong.
+        Normalization:
+
+            type 1 and 3: v.conj() a      v  = w
+
+            type 2: inv(v).conj() a  inv(v) = w
+
+            type = 1 or 2: v.conj() b      v  = I
+
+            type = 3: v.conj() inv(b) v  = I
+
+    Raises
+    ------
+    LinAlgError :
+        If eigenvalue computation does not converge,
+        an error occurred, or b matrix is not definite positive. Note that
+        if input matrices are not symmetric or hermitian, no error is reported
+        but results will be wrong.
 
     See Also
     --------
     eig : eigenvalues and right eigenvectors for non-symmetric arrays
 
     """
-    a1 = asarray_chkfinite(a)
+    if check_finite:
+        a1 = asarray_chkfinite(a)
+    else:
+        a1 = asarray(a)
     if len(a1.shape) != 2 or a1.shape[0] != a1.shape[1]:
         raise ValueError('expected square matrix')
-    overwrite_a = overwrite_a or (_datanotshared(a1, a))
+    overwrite_a = overwrite_a or (_datacopied(a1, a))
     if iscomplexobj(a1):
         cplx = True
     else:
         cplx = False
     if b is not None:
-        b1 = asarray_chkfinite(b)
-        overwrite_b = overwrite_b or _datanotshared(b1, b)
+        if check_finite:
+            b1 = asarray_chkfinite(b)
+        else:
+            b1 = asarray(b)
+        overwrite_b = overwrite_b or _datacopied(b1, b)
         if len(b1.shape) != 2 or b1.shape[0] != b1.shape[1]:
             raise ValueError('expected square matrix')
 
@@ -321,7 +358,7 @@ def eigh(a, b=None, lower=True, eigvals_only=False, overwrite_a=False,
             w, v, info = evr(a1, uplo=uplo, jobz=_job, range="A", il=1,
                              iu=a1.shape[0], overwrite_a=overwrite_a)
         else:
-            (lo, hi)= eigvals
+            (lo, hi) = eigvals
             w_tot, v, info = evr(a1, uplo=uplo, jobz=_job, range="I",
                                  il=lo, iu=hi, overwrite_a=overwrite_a)
             w = w_tot[0:hi-lo+1]
@@ -346,7 +383,7 @@ def eigh(a, b=None, lower=True, eigvals_only=False, overwrite_a=False,
         # Use '*gv' routine if turbo is off and no eigvals are specified
         else:
             (gv,) = get_lapack_funcs((pfx+'gv',), (a1,b1))
-            v, w, info = gv(a1, b1, uplo=uplo, itype= type, jobz=_job,
+            v, w, info = gv(a1, b1, uplo=uplo, itype=type, jobz=_job,
                             overwrite_a=overwrite_a,
                             overwrite_b=overwrite_b)
 
@@ -382,22 +419,26 @@ def eigh(a, b=None, lower=True, eigvals_only=False, overwrite_a=False,
                           " and no eigenvalues or eigenvectors were"
                           " computed." % (info-b1.shape[0]))
 
+
 def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
-               select='a', select_range=None, max_ev = 0):
-    """Solve real symmetric or complex hermitian band matrix eigenvalue problem.
+               select='a', select_range=None, max_ev=0, check_finite=True):
+    """
+    Solve real symmetric or complex hermitian band matrix eigenvalue problem.
 
     Find eigenvalues w and optionally right eigenvectors v of a::
 
         a v[:,i] = w[i] v[:,i]
         v.H v    = identity
 
-    The matrix a is stored in ab either in lower diagonal or upper
+    The matrix a is stored in a_band either in lower diagonal or upper
     diagonal ordered form:
 
-        ab[u + i - j, j] == a[i,j]        (if upper form; i <= j)
-        ab[    i - j, j] == a[i,j]        (if lower form; i >= j)
+        a_band[u + i - j, j] == a[i,j]        (if upper form; i <= j)
+        a_band[    i - j, j] == a[i,j]        (if lower form; i >= j)
 
-    Example of ab (shape of a is (6,6), u=2)::
+    where u is the number of bands above the diagonal.
+
+    Example of a_band (shape of a is (6,6), u=2)::
 
         upper form:
         *   *   a02 a13 a24 a35
@@ -413,16 +454,16 @@ def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
 
     Parameters
     ----------
-    a_band : array, shape (M, u+1)
-        Banded matrix whose eigenvalues to calculate
-    lower : boolean
+    a_band : (u+1, M) array_like
+        The bands of the M by M matrix a.
+    lower : bool, optional
         Is the matrix in the lower form. (Default is upper form)
-    eigvals_only : boolean
+    eigvals_only : bool, optional
         Compute only the eigenvalues and no eigenvectors.
         (Default: calculate also eigenvectors)
-    overwrite_a_band:
+    overwrite_a_band : bool, optional
         Discard data in a_band (may enhance performance)
-    select: {'a', 'v', 'i'}
+    select : {'a', 'v', 'i'}, optional
         Which eigenvalues to calculate
 
         ======  ========================================
@@ -432,21 +473,25 @@ def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
         'v'     Eigenvalues in the interval (min, max]
         'i'     Eigenvalues with indices min <= i <= max
         ======  ========================================
-    select_range : (min, max)
+    select_range : (min, max), optional
         Range of selected eigenvalues
-    max_ev : integer
+    max_ev : int, optional
         For select=='v', maximum number of eigenvalues expected.
         For other values of select, has no meaning.
 
         In doubt, leave this parameter untouched.
 
+    check_finite : boolean, optional
+        Whether to check that the input matrix contains only finite numbers.
+        Disabling may give a performance gain, but may result in problems
+        (crashes, non-termination) if the inputs do contain infinities or NaNs.
+
     Returns
     -------
-    w : array, shape (M,)
+    w : (M,) ndarray
         The eigenvalues, in ascending order, each repeated according to its
         multiplicity.
-
-    v : double or complex double array, shape (M, M)
+    v : (M, M) float or complex ndarray
         The normalized eigenvector corresponding to the eigenvalue w[i] is
         the column v[:,i].
 
@@ -454,8 +499,11 @@ def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
 
     """
     if eigvals_only or overwrite_a_band:
-        a1 = asarray_chkfinite(a_band)
-        overwrite_a_band = overwrite_a_band or (_datanotshared(a1, a_band))
+        if check_finite:
+            a1 = asarray_chkfinite(a_band)
+        else:
+            a1 = asarray(a_band)
+        overwrite_a_band = overwrite_a_band or (_datacopied(a1, a_band))
     else:
         a1 = array(a_band)
         if issubclass(a1.dtype.type, inexact) and not isfinite(a1).all():
@@ -472,13 +520,13 @@ def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
             # FIXME: implement this somewhen, for now go with builtin values
             # FIXME: calc optimal lwork by calling ?hbevd(lwork=-1)
             #        or by using calc_lwork.f ???
-            # lwork = calc_lwork.hbevd(bevd.prefix, a1.shape[0], lower)
+            # lwork = calc_lwork.hbevd(bevd.typecode, a1.shape[0], lower)
             internal_name = 'hbevd'
-        else: # a1.dtype.char in 'fd':
+        else:  # a1.dtype.char in 'fd':
             bevd, = get_lapack_funcs(('sbevd',), (a1,))
             # FIXME: implement this somewhen, for now go with builtin values
             #         see above
-            # lwork = calc_lwork.sbevd(bevd.prefix, a1.shape[0], lower)
+            # lwork = calc_lwork.sbevd(bevd.typecode, a1.shape[0], lower)
             internal_name = 'sbevd'
         w,v,info = bevd(a1, compute_v=not eigvals_only,
                         lower=lower,
@@ -507,7 +555,7 @@ def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
         if a1.dtype.char in 'GFD':
             bevx, = get_lapack_funcs(('hbevx',), (a1,))
             internal_name = 'hbevx'
-        else: # a1.dtype.char in 'gfd'
+        else:  # a1.dtype.char in 'gfd'
             bevx, = get_lapack_funcs(('sbevx',), (a1,))
             internal_name = 'sbevx'
         # il+1, iu+1: translate python indexing (0 ... N-1) into Fortran
@@ -532,8 +580,10 @@ def eig_banded(a_band, lower=False, eigvals_only=False, overwrite_a_band=False,
         return w
     return w, v
 
-def eigvals(a, b=None, overwrite_a=False):
-    """Compute eigenvalues from an ordinary or generalized eigenvalue problem.
+
+def eigvals(a, b=None, overwrite_a=False, check_finite=True):
+    """
+    Compute eigenvalues from an ordinary or generalized eigenvalue problem.
 
     Find eigenvalues of a general matrix::
 
@@ -541,35 +591,46 @@ def eigvals(a, b=None, overwrite_a=False):
 
     Parameters
     ----------
-    a : array, shape (M, M)
+    a : (M, M) array_like
         A complex or real matrix whose eigenvalues and eigenvectors
         will be computed.
-    b : array, shape (M, M)
+    b : (M, M) array_like, optional
         Right-hand side matrix in a generalized eigenvalue problem.
         If omitted, identity matrix is assumed.
-    overwrite_a : boolean
+    overwrite_a : boolean, optional
         Whether to overwrite data in a (may improve performance)
+    check_finite : boolean, optional
+        Whether to check that the input matrices contain only finite numbers.
+        Disabling may give a performance gain, but may result in problems
+        (crashes, non-termination) if the inputs do contain infinities or NaNs.
 
     Returns
     -------
-    w : double or complex array, shape (M,)
+    w : (M,) double or complex ndarray
         The eigenvalues, each repeated according to its multiplicity,
         but not in any specific order.
 
-    Raises LinAlgError if eigenvalue computation does not converge
+    Raises
+    ------
+    LinAlgError
+        If eigenvalue computation does not converge
 
     See Also
     --------
-    eigvalsh : eigenvalues of symmetric or Hemitiean arrays
-    eig : eigenvalues and right eigenvectors of general arrays
-    eigh : eigenvalues and eigenvectors of symmetric/Hermitean arrays.
+    eigvalsh : eigenvalues of symmetric or Hermitian arrays,
+    eig : eigenvalues and right eigenvectors of general arrays.
+    eigh : eigenvalues and eigenvectors of symmetric/Hermitian arrays.
 
     """
-    return eig(a, b=b, left=0, right=0, overwrite_a=overwrite_a)
+    return eig(a, b=b, left=0, right=0, overwrite_a=overwrite_a,
+                check_finite=check_finite)
+
 
 def eigvalsh(a, b=None, lower=True, overwrite_a=False,
-             overwrite_b=False, turbo=True, eigvals=None, type=1):
-    """Solve an ordinary or generalized eigenvalue problem for a complex
+             overwrite_b=False, turbo=True, eigvals=None, type=1,
+             check_finite=True):
+    """
+    Solve an ordinary or generalized eigenvalue problem for a complex
     Hermitian or real symmetric matrix.
 
     Find eigenvalues w of matrix a, where b is positive definite::
@@ -581,42 +642,52 @@ def eigvalsh(a, b=None, lower=True, overwrite_a=False,
 
     Parameters
     ----------
-    a : array, shape (M, M)
+    a : (M, M) array_like
         A complex Hermitian or real symmetric matrix whose eigenvalues and
         eigenvectors will be computed.
-    b : array, shape (M, M)
+    b : (M, M) array_like, optional
         A complex Hermitian or real symmetric definite positive matrix in.
         If omitted, identity matrix is assumed.
-    lower : boolean
+    lower : bool, optional
         Whether the pertinent array data is taken from the lower or upper
-        triangle of a. (Default: lower)
-    turbo : boolean
+        triangle of `a`. (Default: lower)
+    turbo : bool, optional
         Use divide and conquer algorithm (faster but expensive in memory,
         only for generalized eigenvalue problem and if eigvals=None)
-    eigvals : tuple (lo, hi)
+    eigvals : tuple (lo, hi), optional
         Indexes of the smallest and largest (in ascending order) eigenvalues
         and corresponding eigenvectors to be returned: 0 <= lo < hi <= M-1.
         If omitted, all eigenvalues and eigenvectors are returned.
-    type: integer
+    type : integer, optional
         Specifies the problem type to be solved:
+
            type = 1: a   v[:,i] = w[i] b v[:,i]
+
            type = 2: a b v[:,i] = w[i]   v[:,i]
+
            type = 3: b a v[:,i] = w[i]   v[:,i]
-    overwrite_a : boolean
-        Whether to overwrite data in a (may improve performance)
-    overwrite_b : boolean
-        Whether to overwrite data in b (may improve performance)
+    overwrite_a : bool, optional
+        Whether to overwrite data in `a` (may improve performance)
+    overwrite_b : bool, optional
+        Whether to overwrite data in `b` (may improve performance)
+    check_finite : boolean, optional
+        Whether to check that the input matrices contain only finite numbers.
+        Disabling may give a performance gain, but may result in problems
+        (crashes, non-termination) if the inputs do contain infinities or NaNs.
 
     Returns
     -------
-    w : real array, shape (N,)
+    w : (N,) float ndarray
         The N (1<=N<=M) selected eigenvalues, in ascending order, each
         repeated according to its multiplicity.
 
-    Raises LinAlgError if eigenvalue computation does not converge,
-    an error occurred, or b matrix is not definite positive. Note that
-    if input matrices are not symmetric or hermitian, no error is reported
-    but results will be wrong.
+    Raises
+    ------
+    LinAlgError :
+        If eigenvalue computation does not converge,
+        an error occurred, or b matrix is not definite positive. Note that
+        if input matrices are not symmetric or hermitian, no error is reported
+        but results will be wrong.
 
     See Also
     --------
@@ -627,24 +698,29 @@ def eigvalsh(a, b=None, lower=True, overwrite_a=False,
     """
     return eigh(a, b=b, lower=lower, eigvals_only=True,
                 overwrite_a=overwrite_a, overwrite_b=overwrite_b,
-                turbo=turbo, eigvals=eigvals, type=type)
+                turbo=turbo, eigvals=eigvals, type=type,
+                check_finite=check_finite)
+
 
 def eigvals_banded(a_band, lower=False, overwrite_a_band=False,
-                   select='a', select_range=None):
-    """Solve real symmetric or complex hermitian band matrix eigenvalue problem.
+                   select='a', select_range=None, check_finite=True):
+    """
+    Solve real symmetric or complex hermitian band matrix eigenvalue problem.
 
     Find eigenvalues w of a::
 
         a v[:,i] = w[i] v[:,i]
         v.H v    = identity
 
-    The matrix a is stored in ab either in lower diagonal or upper
+    The matrix a is stored in a_band either in lower diagonal or upper
     diagonal ordered form:
 
-        ab[u + i - j, j] == a[i,j]        (if upper form; i <= j)
-        ab[    i - j, j] == a[i,j]        (if lower form; i >= j)
+        a_band[u + i - j, j] == a[i,j]        (if upper form; i <= j)
+        a_band[    i - j, j] == a[i,j]        (if lower form; i >= j)
 
-    Example of ab (shape of a is (6,6), u=2)::
+    where u is the number of bands above the diagonal.
+
+    Example of a_band (shape of a is (6,6), u=2)::
 
         upper form:
         *   *   a02 a13 a24 a35
@@ -660,13 +736,13 @@ def eigvals_banded(a_band, lower=False, overwrite_a_band=False,
 
     Parameters
     ----------
-    a_band : array, shape (M, u+1)
-        Banded matrix whose eigenvalues to calculate
+    a_band : (u+1, M) array_like
+        The bands of the M by M matrix a.
     lower : boolean
         Is the matrix in the lower form. (Default is upper form)
     overwrite_a_band:
         Discard data in a_band (may enhance performance)
-    select: {'a', 'v', 'i'}
+    select : {'a', 'v', 'i'}
         Which eigenvalues to calculate
 
         ======  ========================================
@@ -678,10 +754,14 @@ def eigvals_banded(a_band, lower=False, overwrite_a_band=False,
         ======  ========================================
     select_range : (min, max)
         Range of selected eigenvalues
+    check_finite : boolean, optional
+        Whether to check that the input matrix contains only finite numbers.
+        Disabling may give a performance gain, but may result in problems
+        (crashes, non-termination) if the inputs do contain infinities or NaNs.
 
     Returns
     -------
-    w : array, shape (M,)
+    w : (M,) ndarray
         The eigenvalues, in ascending order, each repeated according to its
         multiplicity.
 
@@ -689,7 +769,8 @@ def eigvals_banded(a_band, lower=False, overwrite_a_band=False,
 
     See Also
     --------
-    eig_banded : eigenvalues and right eigenvectors for symmetric/Hermitian band matrices
+    eig_banded : eigenvalues and right eigenvectors for symmetric/Hermitian
+        band matrices
     eigvals : eigenvalues of general arrays
     eigh : eigenvalues and right eigenvectors for symmetric/Hermitian arrays
     eig : eigenvalues and right eigenvectors for non-symmetric arrays
@@ -697,51 +778,64 @@ def eigvals_banded(a_band, lower=False, overwrite_a_band=False,
     """
     return eig_banded(a_band, lower=lower, eigvals_only=1,
                       overwrite_a_band=overwrite_a_band, select=select,
-                      select_range=select_range)
+                      select_range=select_range, check_finite=check_finite)
 
 _double_precision = ['i','l','d']
 
 
-def hessenberg(a, calc_q=False, overwrite_a=False):
-    """Compute Hessenberg form of a matrix.
+def hessenberg(a, calc_q=False, overwrite_a=False, check_finite=True):
+    """
+    Compute Hessenberg form of a matrix.
 
-    The Hessenberg decomposition is
+    The Hessenberg decomposition is::
 
         A = Q H Q^H
 
-    where Q is unitary/orthogonal and H has only zero elements below the first
-    subdiagonal.
+    where `Q` is unitary/orthogonal and `H` has only zero elements below
+    the first sub-diagonal.
 
     Parameters
     ----------
-    a : array, shape (M,M)
-        Matrix to bring into Hessenberg form
-    calc_q : boolean
-        Whether to compute the transformation matrix
-    overwrite_a : boolean
-        Whether to ovewrite data in a (may improve performance)
+    a : (M, M) array_like
+        Matrix to bring into Hessenberg form.
+    calc_q : bool, optional
+        Whether to compute the transformation matrix.  Default is False.
+    overwrite_a : bool, optional
+        Whether to overwrite `a`; may improve performance.
+        Default is False.
+    check_finite : boolean, optional
+        Whether to check that the input matrix contains only finite numbers.
+        Disabling may give a performance gain, but may result in problems
+        (crashes, non-termination) if the inputs do contain infinities or NaNs.
 
     Returns
     -------
-    H : array, shape (M,M)
-        Hessenberg form of A
-
-    (If calc_q == True)
-    Q : array, shape (M,M)
-        Unitary/orthogonal similarity transformation matrix s.t. A = Q H Q^H
+    H : (M, M) ndarray
+        Hessenberg form of `a`.
+    Q : (M, M) ndarray
+        Unitary/orthogonal similarity transformation matrix ``A = Q H Q^H``.
+        Only returned if ``calc_q=True``.
 
     """
-    a1 = asarray(a)
+    if check_finite:
+        a1 = asarray_chkfinite(a)
+    else:
+        a1 = asarray(a)
     if len(a1.shape) != 2 or (a1.shape[0] != a1.shape[1]):
         raise ValueError('expected square matrix')
-    overwrite_a = overwrite_a or (_datanotshared(a1, a))
-    gehrd,gebal = get_lapack_funcs(('gehrd','gebal'), (a1,))
-    ba, lo, hi, pivscale, info = gebal(a, permute=1, overwrite_a=overwrite_a)
+    overwrite_a = overwrite_a or (_datacopied(a1, a))
+    gehrd, gebal, gehrd_lwork = get_lapack_funcs(('gehrd','gebal', 'gehrd_lwork'), (a1,))
+    ba, lo, hi, pivscale, info = gebal(a1, permute=0, overwrite_a=overwrite_a)
     if info < 0:
         raise ValueError('illegal value in %d-th argument of internal gebal '
                                                     '(hessenberg)' % -info)
     n = len(a1)
-    lwork = calc_lwork.gehrd(gehrd.prefix, n, lo, hi)
+
+    lwork, info = gehrd_lwork(ba.shape[0], lo=lo, hi=hi)
+    if info != 0:
+        raise ValueError('failed to compute internal gehrd work array size' % info)
+    lwork = int(lwork.real)
+
     hq, tau, info = gehrd(ba, lo=lo, hi=hi, lwork=lwork, overwrite_a=1)
     if info < 0:
         raise ValueError('illegal value in %d-th argument of internal gehrd '
@@ -753,11 +847,11 @@ def hessenberg(a, calc_q=False, overwrite_a=False):
         return hq
 
     # XXX: Use ORGHR routines to compute q.
-    ger,gemm = get_blas_funcs(('ger','gemm'), (hq,))
-    typecode = hq.dtype.char
+    typecode = hq.dtype
+    ger,gemm = get_blas_funcs(('ger','gemm'), dtype=typecode)
     q = None
     for i in range(lo, hi):
-        if tau[i]==0.0:
+        if tau[i] == 0.0:
             continue
         v = zeros(n, dtype=typecode)
         v[i+1] = 1.0
