@@ -1,14 +1,19 @@
 """Tests for functions in special_matrices.py."""
 
-from numpy import arange, add, array, eye, copy
+from __future__ import division, print_function, absolute_import
+
+from numpy import arange, add, array, eye, copy, sqrt
 from numpy.testing import TestCase, run_module_suite, assert_raises, \
     assert_equal, assert_array_equal, assert_array_almost_equal, \
     assert_allclose
 
+from scipy.lib.six import xrange
+
 from scipy.misc import comb
 from scipy.linalg import toeplitz, hankel, circulant, hadamard, leslie, \
                             companion, tri, triu, tril, kron, block_diag, \
-                            hilbert, invhilbert, pascal
+                            hilbert, invhilbert, pascal, dft
+from scipy.fftpack import fft
 from numpy.linalg import cond
 
 
@@ -28,6 +33,7 @@ class TestTri(TestCase):
                                                 [1,1,0,0],
                                                 [1,1,1,0],
                                                 [1,1,1,1]],'f'))
+
     def test_diag(self):
         assert_equal(tri(4,k=1),array([[1,1,0,0],
                                        [1,1,1,0],
@@ -37,6 +43,7 @@ class TestTri(TestCase):
                                         [1,0,0,0],
                                         [1,1,0,0],
                                         [1,1,1,0]]))
+
     def test_2d(self):
         assert_equal(tri(4,3),array([[1,0,0],
                                      [1,1,0],
@@ -45,6 +52,7 @@ class TestTri(TestCase):
         assert_equal(tri(3,4),array([[1,0,0,0],
                                      [1,1,0,0],
                                      [1,1,1,0]]))
+
     def test_diag2d(self):
         assert_equal(tri(3,4,k=2),array([[1,1,1,0],
                                          [1,1,1,1],
@@ -54,6 +62,7 @@ class TestTri(TestCase):
                                           [1,0,0],
                                           [1,1,0]]))
 
+
 class TestTril(TestCase):
     def test_basic(self):
         a = (100*get_mat(5)).astype('l')
@@ -189,9 +198,9 @@ class TestLeslie(TestCase):
     def test_basic(self):
         a = leslie([1, 2, 3], [0.25, 0.5])
         expected = array([
-            [1.0,  2.0, 3.0],
+            [1.0, 2.0, 3.0],
             [0.25, 0.0, 0.0],
-            [0.0,  0.5, 0.0]])
+            [0.0, 0.5, 0.0]])
         assert_array_equal(a, expected)
 
 
@@ -207,13 +216,13 @@ class TestCompanion(TestCase):
         c = companion([1, 2, 3])
         expected = array([
             [-2.0, -3.0],
-            [ 1.0,  0.0]])
+            [1.0, 0.0]])
         assert_array_equal(c, expected)
 
         c = companion([2.0, 5.0, -10.0])
         expected = array([
             [-2.5, 5.0],
-            [ 1.0, 0.0]])
+            [1.0, 0.0]])
         assert_array_equal(c, expected)
 
 
@@ -262,17 +271,17 @@ class TestKron:
         m1 = array([[1, 2], [3, 4]])
         m2 = array([[10], [11]])
         a = kron(m1, m2)
-        expected = array([[ 10, 20 ],
-                          [ 11, 22 ],
-                          [ 30, 40 ],
-                          [ 33, 44 ]])
+        expected = array([[10, 20],
+                          [11, 22],
+                          [30, 40],
+                          [33, 44]])
         assert_array_equal(a, expected)
 
 
 class TestHilbert(TestCase):
 
     def test_basic(self):
-        h3 = array([[1.0,  1/2., 1/3.],
+        h3 = array([[1.0, 1/2., 1/3.],
                     [1/2., 1/3., 1/4.],
                     [1/3., 1/4., 1/5.]])
         assert_array_almost_equal(hilbert(3), h3)
@@ -290,29 +299,29 @@ class TestInvHilbert(TestCase):
         assert_array_equal(invhilbert(1, exact=True), invh1)
         assert_array_equal(invhilbert(1), invh1)
 
-        invh2 = array([[ 4, -6],
+        invh2 = array([[4, -6],
                        [-6, 12]])
         assert_array_equal(invhilbert(2, exact=True), invh2)
         assert_array_almost_equal(invhilbert(2), invh2)
 
-        invh3 = array([[  9,  -36,  30],
-                       [-36,  192, -180],
-                        [30, -180,  180]])
+        invh3 = array([[9, -36, 30],
+                       [-36, 192, -180],
+                        [30, -180, 180]])
         assert_array_equal(invhilbert(3, exact=True), invh3)
         assert_array_almost_equal(invhilbert(3), invh3)
 
-        invh4 = array([[  16,  -120,   240,  -140],
-                       [-120,  1200, -2700,  1680],
-                       [ 240, -2700,  6480, -4200],
-                       [-140,  1680, -4200,  2800]])
+        invh4 = array([[16, -120, 240, -140],
+                       [-120, 1200, -2700, 1680],
+                       [240, -2700, 6480, -4200],
+                       [-140, 1680, -4200, 2800]])
         assert_array_equal(invhilbert(4, exact=True), invh4)
         assert_array_almost_equal(invhilbert(4), invh4)
 
-        invh5 = array([[   25,   -300,    1050,  -1400,     630],
-                       [ -300,   4800,  -18900,   26880, -12600],
-                       [ 1050, -18900,   79380, -117600,  56700],
-                       [-1400,  26880, -117600,  179200, -88200],
-                       [  630, -12600,   56700,  -88200,  44100]])
+        invh5 = array([[25, -300, 1050, -1400, 630],
+                       [-300, 4800, -18900, 26880, -12600],
+                       [1050, -18900, 79380, -117600, 56700],
+                       [-1400, 26880, -117600, 179200, -88200],
+                       [630, -12600, 56700, -88200, 44100]])
         assert_array_equal(invhilbert(5, exact=True), invh5)
         assert_array_almost_equal(invhilbert(5), invh5)
 
@@ -453,10 +462,10 @@ class TestPascal(TestCase):
             array([[1, 0, 0],
                    [1, 1, 0],
                    [1, 2, 1]])),
-        (4, array([[1,  1,  1,  1],
-                   [1,  2,  3,  4],
-                   [1,  3,  6, 10],
-                   [1,  4, 10, 20]]),
+        (4, array([[1, 1, 1, 1],
+                   [1, 2, 3, 4],
+                   [1, 3, 6, 10],
+                   [1, 4, 10, 20]]),
             array([[1, 0, 0, 0],
                    [1, 1, 0, 0],
                    [1, 2, 1, 0],
@@ -479,6 +488,32 @@ class TestPascal(TestCase):
         p = pascal(50)
         assert_equal(p[-1, -1], comb(98, 49, exact=True))
 
+    def test_threshold(self):
+        # Regression test.  An early version of `pascal` returned an
+        # array of type np.uint64 for n=35, but that data type is too small
+        # to hold p[-1, -1].  The second assert_equal below would fail
+        # because p[-1, -1] overflowed.
+        p = pascal(34)
+        assert_equal(2*p.item(-1, -2), p.item(-1, -1), err_msg="n = 34")
+        p = pascal(35)
+        assert_equal(2*p.item(-1, -2), p.item(-1, -1), err_msg="n = 35")
+
+
+def test_dft():
+    m = dft(2)
+    expected = array([[1.0, 1.0], [1.0, -1.0]])
+    yield (assert_array_almost_equal, m, expected)
+    m = dft(2, scale='n')
+    yield (assert_array_almost_equal, m, expected/2.0)
+    m = dft(2, scale='sqrtn')
+    yield (assert_array_almost_equal, m, expected/sqrt(2.0))
+
+    x = array([0, 1, 2, 3, 4, 5, 0, 1])
+    m = dft(8)
+    mx = m.dot(x)
+    fx = fft(x)
+    yield (assert_array_almost_equal, mx, fx)
+
 
 if __name__ == "__main__":
     run_module_suite()
