@@ -28,11 +28,14 @@
 # NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+from __future__ import division, print_function, absolute_import
+
 import math
 import numpy
-import _ni_support
-import _nd_image
+from . import _ni_support
+from . import _nd_image
 from scipy.misc import doccer
+from scipy.lib._version import NumpyVersion
 
 __all__ = ['correlate1d', 'convolve1d', 'gaussian_filter1d', 'gaussian_filter',
            'prewitt', 'sobel', 'generic_laplace', 'laplace',
@@ -45,42 +48,43 @@ __all__ = ['correlate1d', 'convolve1d', 'gaussian_filter1d', 'gaussian_filter',
 
 
 _input_doc = \
-"""input : array-like
-    input array to filter"""
+"""input : array_like
+    Input array to filter."""
 _axis_doc = \
-"""axis : integer, optional
-    axis of ``input`` along which to calculate. Default is -1"""
+"""axis : int, optional
+    The axis of `input` along which to calculate. Default is -1."""
 _output_doc = \
 """output : array, optional
-    The ``output`` parameter passes an array in which to store the
+    The `output` parameter passes an array in which to store the
     filter output."""
 _size_foot_doc = \
 """size : scalar or tuple, optional
     See footprint, below
 footprint : array, optional
-    Either ``size`` or ``footprint`` must be defined.  ``size`` gives
+    Either `size` or `footprint` must be defined.  `size` gives
     the shape that is taken from the input array, at every element
     position, to define the input to the filter function.
-    ``footprint`` is a boolean array that specifies (implicitly) a
+    `footprint` is a boolean array that specifies (implicitly) a
     shape, but also which of the elements within this shape will get
     passed to the filter function.  Thus ``size=(n,m)`` is equivalent
-    to ``footprint=np.ones((n,m))``.  We adjust ``size`` to the number
+    to ``footprint=np.ones((n,m))``.  We adjust `size` to the number
     of dimensions of the input array, so that, if the input array is
-    shape (10,10,10), and ``size`` is 2, then the actual size used is
+    shape (10,10,10), and `size` is 2, then the actual size used is
     (2,2,2).
 """
 _mode_doc = \
-"""mode : {'reflect','constant','nearest','mirror', 'wrap'}, optional
-    The ``mode`` parameter determines how the array borders are
-    handled, where ``cval`` is the value when mode is equal to
+"""mode : {'reflect', 'constant', 'nearest', 'mirror', 'wrap'}, optional
+    The `mode` parameter determines how the array borders are
+    handled, where `cval` is the value when mode is equal to
     'constant'. Default is 'reflect'"""
 _cval_doc = \
 """cval : scalar, optional
-    Value to fill past edges of input if ``mode`` is 'constant'. Default
+    Value to fill past edges of input if `mode` is 'constant'. Default
     is 0.0"""
 _origin_doc = \
 """origin : scalar, optional
-The ``origin`` parameter controls the placement of the filter. Default 0"""
+    The `origin` parameter controls the placement of the filter.
+    Default 0.0."""
 _extra_arguments_doc = \
 """extra_arguments : sequence, optional
     Sequence of extra positional arguments to pass to passed function"""
@@ -89,22 +93,23 @@ _extra_keywords_doc = \
     dict of extra keyword arguments to pass to passed function"""
 
 docdict = {
-    'input':_input_doc,
-    'axis':_axis_doc,
-    'output':_output_doc,
-    'size_foot':_size_foot_doc,
-    'mode':_mode_doc,
-    'cval':_cval_doc,
-    'origin':_origin_doc,
-    'extra_arguments':_extra_arguments_doc,
-    'extra_keywords':_extra_keywords_doc,
+    'input': _input_doc,
+    'axis': _axis_doc,
+    'output': _output_doc,
+    'size_foot': _size_foot_doc,
+    'mode': _mode_doc,
+    'cval': _cval_doc,
+    'origin': _origin_doc,
+    'extra_arguments': _extra_arguments_doc,
+    'extra_keywords': _extra_keywords_doc,
     }
 
 docfiller = doccer.filldoc(docdict)
 
+
 @docfiller
-def correlate1d(input, weights, axis = -1, output = None, mode = "reflect",
-                cval = 0.0, origin = 0):
+def correlate1d(input, weights, axis=-1, output=None, mode="reflect",
+                cval=0.0, origin=0):
     """Calculate a one-dimensional correlation along the given axis.
 
     The lines of the array along the given axis are correlated with the
@@ -114,7 +119,7 @@ def correlate1d(input, weights, axis = -1, output = None, mode = "reflect",
     ----------
     %(input)s
     weights : array
-        one-dimensional sequence of numbers
+        One-dimensional sequence of numbers.
     %(axis)s
     %(output)s
     %(mode)s
@@ -131,8 +136,8 @@ def correlate1d(input, weights, axis = -1, output = None, mode = "reflect",
     if not weights.flags.contiguous:
         weights = weights.copy()
     axis = _ni_support._check_axis(axis, input.ndim)
-    if ((len(weights) // 2 + origin < 0) or
-        (len(weights) // 2 + origin > len(weights))):
+    if (len(weights) // 2 + origin < 0) or (len(weights) // 2 +
+                                            origin > len(weights)):
         raise ValueError('invalid origin')
     mode = _ni_support._extend_mode_to_code(mode)
     _nd_image.correlate1d(input, weights, axis, output, mode, cval,
@@ -141,8 +146,8 @@ def correlate1d(input, weights, axis = -1, output = None, mode = "reflect",
 
 
 @docfiller
-def convolve1d(input, weights, axis = -1, output = None, mode = "reflect",
-               cval = 0.0, origin = 0):
+def convolve1d(input, weights, axis=-1, output=None, mode="reflect",
+               cval=0.0, origin=0):
     """Calculate a one-dimensional convolution along the given axis.
 
     The lines of the array along the given axis are convolved with the
@@ -152,12 +157,18 @@ def convolve1d(input, weights, axis = -1, output = None, mode = "reflect",
     ----------
     %(input)s
     weights : ndarray
-        one-dimensional sequence of numbers
+        One-dimensional sequence of numbers.
     %(axis)s
     %(output)s
     %(mode)s
     %(cval)s
     %(origin)s
+
+    Returns
+    -------
+    convolve1d : ndarray
+        Convolved array with same shape as input
+
     """
     weights = weights[::-1]
     origin = -origin
@@ -167,8 +178,8 @@ def convolve1d(input, weights, axis = -1, output = None, mode = "reflect",
 
 
 @docfiller
-def gaussian_filter1d(input, sigma, axis = -1, order = 0, output = None,
-                      mode = "reflect", cval = 0.0):
+def gaussian_filter1d(input, sigma, axis=-1, order=0, output=None,
+                      mode="reflect", cval=0.0, truncate=4.0):
     """One-dimensional Gaussian filter.
 
     Parameters
@@ -185,13 +196,20 @@ def gaussian_filter1d(input, sigma, axis = -1, order = 0, output = None,
     %(output)s
     %(mode)s
     %(cval)s
+    truncate : float
+        Truncate the filter at this many standard deviations.
+        Default is 4.0.
+
+    Returns
+    -------
+    gaussian_filter1d : ndarray
+
     """
     if order not in range(4):
         raise ValueError('Order outside 0..3 not implemented')
     sd = float(sigma)
-    # make the length of the filter equal to 4 times the standard
-    # deviations:
-    lw = int(4.0 * sd + 0.5)
+    # make the radius of the filter equal to truncate standard deviations
+    lw = int(truncate * sd + 0.5)
     weights = [0.0] * (2 * lw + 1)
     weights[lw] = 1.0
     sum = 1.0
@@ -205,21 +223,21 @@ def gaussian_filter1d(input, sigma, axis = -1, order = 0, output = None,
     for ii in range(2 * lw + 1):
         weights[ii] /= sum
     # implement first, second and third order derivatives:
-    if order == 1 : # first derivative
+    if order == 1:  # first derivative
         weights[lw] = 0.0
         for ii in range(1, lw + 1):
             x = float(ii)
             tmp = -x / sd * weights[lw + ii]
             weights[lw + ii] = -tmp
             weights[lw - ii] = tmp
-    elif order == 2: # second derivative
+    elif order == 2:  # second derivative
         weights[lw] *= -1.0 / sd
         for ii in range(1, lw + 1):
             x = float(ii)
             tmp = (x * x / sd - 1.0) * weights[lw + ii] / sd
             weights[lw + ii] = tmp
             weights[lw - ii] = tmp
-    elif order == 3: # third derivative
+    elif order == 3:  # third derivative
         weights[lw] = 0.0
         sd2 = sd * sd
         for ii in range(1, lw + 1):
@@ -231,15 +249,15 @@ def gaussian_filter1d(input, sigma, axis = -1, order = 0, output = None,
 
 
 @docfiller
-def gaussian_filter(input, sigma, order = 0, output = None,
-                  mode = "reflect", cval = 0.0):
-    """Multi-dimensional Gaussian filter.
+def gaussian_filter(input, sigma, order=0, output=None,
+                  mode="reflect", cval=0.0, truncate=4.0):
+    """Multidimensional Gaussian filter.
 
     Parameters
     ----------
     %(input)s
     sigma : scalar or sequence of scalars
-        standard deviation for Gaussian kernel. The standard
+        Standard deviation for Gaussian kernel. The standard
         deviations of the Gaussian filter are given for each axis as a
         sequence, or as a single number, in which case it is equal for
         all axes.
@@ -253,15 +271,24 @@ def gaussian_filter(input, sigma, order = 0, output = None,
     %(output)s
     %(mode)s
     %(cval)s
+    truncate : float
+        Truncate the filter at this many standard deviations.
+        Default is 4.0.
+
+    Returns
+    -------
+    gaussian_filter : ndarray
+        Returned array of same shape as `input`.
 
     Notes
     -----
-    The multi-dimensional filter is implemented as a sequence of
+    The multidimensional filter is implemented as a sequence of
     one-dimensional convolution filters. The intermediate arrays are
     stored in the same data type as the output. Therefore, for output
     types with a limited precision, the results may be imprecise
     because intermediate results may be stored with insufficient
     precision.
+
     """
     input = numpy.asarray(input)
     output, return_value = _ni_support._get_output(output, input)
@@ -269,13 +296,13 @@ def gaussian_filter(input, sigma, order = 0, output = None,
     if not set(orders).issubset(set(range(4))):
         raise ValueError('Order outside 0..4 not implemented')
     sigmas = _ni_support._normalize_sequence(sigma, input.ndim)
-    axes = range(input.ndim)
+    axes = list(range(input.ndim))
     axes = [(axes[ii], sigmas[ii], orders[ii])
                         for ii in range(len(axes)) if sigmas[ii] > 1e-15]
     if len(axes) > 0:
         for axis, sigma, order in axes:
             gaussian_filter1d(input, sigma, axis, order, output,
-                              mode, cval)
+                              mode, cval, truncate)
             input = output
     else:
         output[...] = input[...]
@@ -283,7 +310,7 @@ def gaussian_filter(input, sigma, order = 0, output = None,
 
 
 @docfiller
-def prewitt(input, axis = -1, output = None, mode = "reflect", cval = 0.0):
+def prewitt(input, axis=-1, output=None, mode="reflect", cval=0.0):
     """Calculate a Prewitt filter.
 
     Parameters
@@ -305,7 +332,7 @@ def prewitt(input, axis = -1, output = None, mode = "reflect", cval = 0.0):
 
 
 @docfiller
-def sobel(input, axis = -1, output = None, mode = "reflect", cval = 0.0):
+def sobel(input, axis=-1, output=None, mode="reflect", cval=0.0):
     """Calculate a Sobel filter.
 
     Parameters
@@ -327,12 +354,11 @@ def sobel(input, axis = -1, output = None, mode = "reflect", cval = 0.0):
 
 
 @docfiller
-def generic_laplace(input, derivative2, output = None, mode = "reflect",
-                    cval = 0.0,
-                    extra_arguments = (),
+def generic_laplace(input, derivative2, output=None, mode="reflect",
+                    cval=0.0,
+                    extra_arguments=(),
                     extra_keywords = None):
-    """Calculate a multidimensional laplace filter using the provided
-    second derivative function.
+    """N-dimensional Laplace filter using a provided second derivative function
 
     Parameters
     ----------
@@ -354,7 +380,7 @@ def generic_laplace(input, derivative2, output = None, mode = "reflect",
         extra_keywords = {}
     input = numpy.asarray(input)
     output, return_value = _ni_support._get_output(output, input)
-    axes = range(input.ndim)
+    axes = list(range(input.ndim))
     if len(axes) > 0:
         derivative2(input, axes[0], output, mode, cval,
                     *extra_arguments, **extra_keywords)
@@ -368,9 +394,8 @@ def generic_laplace(input, derivative2, output = None, mode = "reflect",
 
 
 @docfiller
-def laplace(input, output = None, mode = "reflect", cval = 0.0):
-    """Calculate a multidimensional laplace filter using an estimation
-    for the second derivative based on differences.
+def laplace(input, output=None, mode="reflect", cval=0.0):
+    """N-dimensional Laplace filter based on approximate second derivatives.
 
     Parameters
     ----------
@@ -385,10 +410,9 @@ def laplace(input, output = None, mode = "reflect", cval = 0.0):
 
 
 @docfiller
-def gaussian_laplace(input, sigma, output = None, mode = "reflect",
-                     cval = 0.0):
-    """Calculate a multidimensional laplace filter using gaussian
-    second derivatives.
+def gaussian_laplace(input, sigma, output=None, mode="reflect",
+                     cval=0.0, **kwargs):
+    """Multidimensional Laplace filter using gaussian second derivatives.
 
     Parameters
     ----------
@@ -396,26 +420,30 @@ def gaussian_laplace(input, sigma, output = None, mode = "reflect",
     sigma : scalar or sequence of scalars
         The standard deviations of the Gaussian filter are given for
         each axis as a sequence, or as a single number, in which case
-        it is equal for all axes..
+        it is equal for all axes.
     %(output)s
     %(mode)s
     %(cval)s
+    Extra keyword arguments will be passed to gaussian_filter().
     """
     input = numpy.asarray(input)
-    def derivative2(input, axis, output, mode, cval, sigma):
+
+    def derivative2(input, axis, output, mode, cval, sigma, **kwargs):
         order = [0] * input.ndim
         order[axis] = 2
-        return gaussian_filter(input, sigma, order, output, mode, cval)
+        return gaussian_filter(input, sigma, order, output, mode, cval,
+                               **kwargs)
+
     return generic_laplace(input, derivative2, output, mode, cval,
-                           extra_arguments = (sigma,))
+                           extra_arguments=(sigma,),
+                           extra_keywords=kwargs)
 
 
 @docfiller
-def generic_gradient_magnitude(input, derivative, output = None,
-                mode = "reflect", cval = 0.0,
-                extra_arguments = (), extra_keywords = None):
-    """Calculate a gradient magnitude using the provided function for
-    the gradient.
+def generic_gradient_magnitude(input, derivative, output=None,
+                mode="reflect", cval=0.0,
+                extra_arguments=(), extra_keywords = None):
+    """Gradient magnitude using a provided gradient function.
 
     Parameters
     ----------
@@ -424,7 +452,7 @@ def generic_gradient_magnitude(input, derivative, output = None,
         Callable with the following signature::
 
             derivative(input, axis, output, mode, cval,
-                        *extra_arguments, **extra_keywords)
+                       *extra_arguments, **extra_keywords)
 
         See `extra_arguments`, `extra_keywords` below.
         `derivative` can assume that `input` and `output` are ndarrays.
@@ -440,7 +468,7 @@ def generic_gradient_magnitude(input, derivative, output = None,
         extra_keywords = {}
     input = numpy.asarray(input)
     output, return_value = _ni_support._get_output(output, input)
-    axes = range(input.ndim)
+    axes = list(range(input.ndim))
     if len(axes) > 0:
         derivative(input, axes[0], output, mode, cval,
                    *extra_arguments, **extra_keywords)
@@ -451,7 +479,7 @@ def generic_gradient_magnitude(input, derivative, output = None,
             numpy.multiply(tmp, tmp, tmp)
             output += tmp
         # This allows the sqrt to work with a different default casting
-        if numpy.version.short_version > '1.6.1':
+        if NumpyVersion(numpy.__version__) > '1.6.1':
             numpy.sqrt(output, output, casting='unsafe')
         else:
             numpy.sqrt(output, output)
@@ -461,10 +489,9 @@ def generic_gradient_magnitude(input, derivative, output = None,
 
 
 @docfiller
-def gaussian_gradient_magnitude(input, sigma, output = None,
-                mode = "reflect", cval = 0.0):
-    """Calculate a multidimensional gradient magnitude using gaussian
-    derivatives.
+def gaussian_gradient_magnitude(input, sigma, output=None,
+                mode="reflect", cval=0.0, **kwargs):
+    """Multidimensional gradient magnitude using Gaussian derivatives.
 
     Parameters
     ----------
@@ -476,20 +503,25 @@ def gaussian_gradient_magnitude(input, sigma, output = None,
     %(output)s
     %(mode)s
     %(cval)s
+    Extra keyword arguments will be passed to gaussian_filter().
     """
     input = numpy.asarray(input)
-    def derivative(input, axis, output, mode, cval, sigma):
+
+    def derivative(input, axis, output, mode, cval, sigma, **kwargs):
         order = [0] * input.ndim
         order[axis] = 1
-        return gaussian_filter(input, sigma, order, output, mode, cval)
+        return gaussian_filter(input, sigma, order, output, mode,
+                               cval, **kwargs)
+
     return generic_gradient_magnitude(input, derivative, output, mode,
-                            cval, extra_arguments = (sigma,))
+                                      cval, extra_arguments=(sigma,),
+                                      extra_keywords=kwargs)
 
 
 def _correlate_or_convolve(input, weights, output, mode, cval, origin,
                            convolution):
     input = numpy.asarray(input)
-    if numpy.iscomplexobj(int):
+    if numpy.iscomplexobj(input):
         raise TypeError('Complex type not supported')
     origins = _ni_support._normalize_sequence(origin, input.ndim)
     weights = numpy.asarray(weights, dtype=numpy.float64)
@@ -514,8 +546,8 @@ def _correlate_or_convolve(input, weights, output, mode, cval, origin,
 
 
 @docfiller
-def correlate(input, weights, output = None, mode = 'reflect', cval = 0.0,
-              origin = 0):
+def correlate(input, weights, output=None, mode='reflect', cval=0.0,
+              origin=0):
     """
     Multi-dimensional correlation.
 
@@ -551,10 +583,10 @@ def correlate(input, weights, output = None, mode = 'reflect', cval = 0.0,
 
 
 @docfiller
-def convolve(input, weights, output = None, mode = 'reflect', cval = 0.0,
-             origin = 0):
+def convolve(input, weights, output=None, mode='reflect', cval=0.0,
+             origin=0):
     """
-    Multi-dimensional convolution.
+    Multidimensional convolution.
 
     The array is convolved with the given kernel.
 
@@ -574,7 +606,7 @@ def convolve(input, weights, output = None, mode = 'reflect', cval = 0.0,
     cval : scalar, optional
         Value to fill past edges of input if `mode` is 'constant'. Default
         is 0.0
-    origin : scalar, optional
+    origin : array_like, optional
         The `origin` parameter controls the placement of the filter.
         Default is 0.
 
@@ -665,8 +697,8 @@ def convolve(input, weights, output = None, mode = 'reflect', cval = 0.0,
 
 
 @docfiller
-def uniform_filter1d(input, size, axis = -1, output = None,
-                     mode = "reflect", cval = 0.0, origin = 0):
+def uniform_filter1d(input, size, axis=-1, output=None,
+                     mode="reflect", cval=0.0, origin=0):
     """Calculate a one-dimensional uniform filter along the given axis.
 
     The lines of the array along the given axis are filtered with a
@@ -699,8 +731,8 @@ def uniform_filter1d(input, size, axis = -1, output = None,
 
 
 @docfiller
-def uniform_filter(input, size = 3, output = None, mode = "reflect",
-                   cval = 0.0, origin = 0):
+def uniform_filter(input, size=3, output=None, mode="reflect",
+                   cval=0.0, origin=0):
     """Multi-dimensional uniform filter.
 
     Parameters
@@ -727,7 +759,7 @@ def uniform_filter(input, size = 3, output = None, mode = "reflect",
     output, return_value = _ni_support._get_output(output, input)
     sizes = _ni_support._normalize_sequence(size, input.ndim)
     origins = _ni_support._normalize_sequence(origin, input.ndim)
-    axes = range(input.ndim)
+    axes = list(range(input.ndim))
     axes = [(axes[ii], sizes[ii], origins[ii])
                            for ii in range(len(axes)) if sizes[ii] > 1]
     if len(axes) > 0:
@@ -741,8 +773,8 @@ def uniform_filter(input, size = 3, output = None, mode = "reflect",
 
 
 @docfiller
-def minimum_filter1d(input, size, axis = -1, output = None,
-                     mode = "reflect", cval = 0.0, origin = 0):
+def minimum_filter1d(input, size, axis=-1, output=None,
+                     mode="reflect", cval=0.0, origin=0):
     """Calculate a one-dimensional minimum filter along the given axis.
 
     The lines of the array along the given axis are filtered with a
@@ -758,6 +790,17 @@ def minimum_filter1d(input, size, axis = -1, output = None,
     %(mode)s
     %(cval)s
     %(origin)s
+
+    Notes
+    -----
+    This function implements the MINLIST algorithm [1]_, as described by
+    Richard Harter [2]_, and has a guaranteed O(n) performance, `n` being
+    the `input` length, regardless of filter size.
+
+    References
+    ----------
+    .. [1] http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.42.2777
+    .. [2] http://www.richardhartersworld.com/cri/2001/slidingmin.html
     """
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
@@ -775,23 +818,41 @@ def minimum_filter1d(input, size, axis = -1, output = None,
 
 
 @docfiller
-def maximum_filter1d(input, size, axis = -1, output = None,
-                     mode = "reflect", cval = 0.0, origin = 0):
+def maximum_filter1d(input, size, axis=-1, output=None,
+                     mode="reflect", cval=0.0, origin=0):
     """Calculate a one-dimensional maximum filter along the given axis.
 
     The lines of the array along the given axis are filtered with a
     maximum filter of given size.
 
-        Parameters
+    Parameters
     ----------
     %(input)s
     size : int
-        length along which to calculate 1D maximum
+        Length along which to calculate the 1-D maximum.
     %(axis)s
     %(output)s
     %(mode)s
     %(cval)s
     %(origin)s
+
+    Returns
+    -------
+    maximum1d : ndarray, None
+        Maximum-filtered array with same shape as input.
+        None if `output` is not None
+
+    Notes
+    -----
+    This function implements the MAXLIST algorithm [1]_, as described by
+    Richard Harter [2]_, and has a guaranteed O(n) performance, `n` being
+    the `input` length, regardless of filter size.
+
+    References
+    ----------
+    .. [1] http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.42.2777
+    .. [2] http://www.richardhartersworld.com/cri/2001/slidingmin.html
+
     """
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
@@ -814,11 +875,11 @@ def _min_or_max_filter(input, size, footprint, structure, output, mode,
         if footprint is None:
             if size is None:
                 raise RuntimeError("no footprint provided")
-            separable= True
+            separable = True
         else:
             footprint = numpy.asarray(footprint)
             footprint = footprint.astype(bool)
-            if numpy.alltrue(numpy.ravel(footprint),axis=0):
+            if numpy.alltrue(numpy.ravel(footprint), axis=0):
                 size = footprint.shape
                 footprint = None
                 separable = True
@@ -839,7 +900,7 @@ def _min_or_max_filter(input, size, footprint, structure, output, mode,
     origins = _ni_support._normalize_sequence(origin, input.ndim)
     if separable:
         sizes = _ni_support._normalize_sequence(size, input.ndim)
-        axes = range(input.ndim)
+        axes = list(range(input.ndim))
         axes = [(axes[ii], sizes[ii], origins[ii])
                                for ii in range(len(axes)) if sizes[ii] > 1]
         if minimum:
@@ -873,8 +934,8 @@ def _min_or_max_filter(input, size, footprint, structure, output, mode,
 
 
 @docfiller
-def minimum_filter(input, size = None, footprint = None, output = None,
-      mode = "reflect", cval = 0.0, origin = 0):
+def minimum_filter(input, size=None, footprint=None, output=None,
+      mode="reflect", cval=0.0, origin=0):
     """Calculates a multi-dimensional minimum filter.
 
     Parameters
@@ -891,8 +952,8 @@ def minimum_filter(input, size = None, footprint = None, output = None,
 
 
 @docfiller
-def maximum_filter(input, size = None, footprint = None, output = None,
-      mode = "reflect", cval = 0.0, origin = 0):
+def maximum_filter(input, size=None, footprint=None, output=None,
+      mode="reflect", cval=0.0, origin=0):
     """Calculates a multi-dimensional maximum filter.
 
     Parameters
@@ -909,8 +970,8 @@ def maximum_filter(input, size = None, footprint = None, output = None,
 
 
 @docfiller
-def _rank_filter(input, rank, size = None, footprint = None, output = None,
-     mode = "reflect", cval = 0.0, origin = 0, operation = 'rank'):
+def _rank_filter(input, rank, size=None, footprint=None, output=None,
+     mode="reflect", cval=0.0, origin=0, operation='rank'):
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
         raise TypeError('Complex type not supported')
@@ -945,14 +1006,14 @@ def _rank_filter(input, rank, size = None, footprint = None, output = None,
             rank = int(float(filter_size) * percentile / 100.0)
     if rank < 0:
         rank += filter_size
-    if rank < 0  or rank >= filter_size:
+    if rank < 0 or rank >= filter_size:
         raise RuntimeError('rank not within filter footprint size')
     if rank == 0:
         return minimum_filter(input, None, footprint, output, mode, cval,
-                              origin)
+                              origins)
     elif rank == filter_size - 1:
         return maximum_filter(input, None, footprint, output, mode, cval,
-                              origin)
+                              origins)
     else:
         output, return_value = _ni_support._get_output(output, input)
         mode = _ni_support._extend_mode_to_code(mode)
@@ -962,8 +1023,8 @@ def _rank_filter(input, rank, size = None, footprint = None, output = None,
 
 
 @docfiller
-def rank_filter(input, rank, size = None, footprint = None, output = None,
-      mode = "reflect", cval = 0.0, origin = 0):
+def rank_filter(input, rank, size=None, footprint=None, output=None,
+      mode="reflect", cval=0.0, origin=0):
     """Calculates a multi-dimensional rank filter.
 
     Parameters
@@ -983,41 +1044,24 @@ def rank_filter(input, rank, size = None, footprint = None, output = None,
 
 
 @docfiller
-def median_filter(input, size = None, footprint = None, output = None,
-      mode = "reflect", cval = 0.0, origin = 0):
+def median_filter(input, size=None, footprint=None, output=None,
+                  mode="reflect", cval=0.0, origin=0):
     """
-    Calculates a multi-dimensional median filter.
+    Calculates a multidimensional median filter.
 
     Parameters
     ----------
-    input : array-like
-        input array to filter
-    size : scalar or tuple, optional
-        See footprint, below
-    footprint : array, optional
-        Either ``size`` or ``footprint`` must be defined.  ``size`` gives
-        the shape that is taken from the input array, at every element
-        position, to define the input to the filter function.
-        ``footprint`` is a boolean array that specifies (implicitly) a
-        shape, but also which of the elements within this shape will get
-        passed to the filter function.  Thus ``size=(n,m)`` is equivalent
-        to ``footprint=np.ones((n,m))``.  We adjust ``size`` to the number
-        of dimensions of the input array, so that, if the input array is
-        shape (10,10,10), and ``size`` is 2, then the actual size used is
-        (2,2,2).
-    output : array, optional
-        The ``output`` parameter passes an array in which to store the
-        filter output.
-    mode : {'reflect','constant','nearest','mirror', 'wrap'}, optional
-        The ``mode`` parameter determines how the array borders are
-        handled, where ``cval`` is the value when mode is equal to
-        'constant'. Default is 'reflect'
-    cval : scalar, optional
-        Value to fill past edges of input if ``mode`` is 'constant'. Default
-        is 0.0
-    origin : scalar, optional
-        The ``origin`` parameter controls the placement of the filter.
-        Default 0
+    %(input)s
+    %(size_foot)s
+    %(output)s
+    %(mode)s
+    %(cval)s
+    %(origin)s
+
+    Returns
+    -------
+    median_filter : ndarray
+        Return of same shape as `input`.
 
     """
     return _rank_filter(input, 0, size, footprint, output, mode, cval,
@@ -1025,8 +1069,8 @@ def median_filter(input, size = None, footprint = None, output = None,
 
 
 @docfiller
-def percentile_filter(input, percentile, size = None, footprint = None,
-                 output = None, mode = "reflect", cval = 0.0, origin = 0):
+def percentile_filter(input, percentile, size=None, footprint=None,
+                 output=None, mode="reflect", cval=0.0, origin=0):
     """Calculates a multi-dimensional percentile filter.
 
     Parameters
@@ -1046,12 +1090,12 @@ def percentile_filter(input, percentile, size = None, footprint = None,
 
 
 @docfiller
-def generic_filter1d(input, function, filter_size, axis = -1,
-                 output = None, mode = "reflect", cval = 0.0, origin = 0,
-                 extra_arguments = (), extra_keywords = None):
+def generic_filter1d(input, function, filter_size, axis=-1,
+                 output=None, mode="reflect", cval=0.0, origin=0,
+                 extra_arguments=(), extra_keywords = None):
     """Calculate a one-dimensional filter along the given axis.
 
-    generic_filter1d iterates over the lines of the array, calling the
+    `generic_filter1d` iterates over the lines of the array, calling the
     given function at each line. The arguments of the line are the
     input line, and the output line. The input and output lines are 1D
     double arrays.  The input line is extended appropriately according
@@ -1062,9 +1106,9 @@ def generic_filter1d(input, function, filter_size, axis = -1,
     ----------
     %(input)s
     function : callable
-        function to apply along given axis
+        Function to apply along given axis.
     filter_size : scalar
-        length of the filter
+        Length of the filter.
     %(axis)s
     %(output)s
     %(mode)s
@@ -1082,8 +1126,8 @@ def generic_filter1d(input, function, filter_size, axis = -1,
     if filter_size < 1:
         raise RuntimeError('invalid filter size')
     axis = _ni_support._check_axis(axis, input.ndim)
-    if ((filter_size // 2 + origin < 0) or
-        (filter_size // 2 + origin >= filter_size)):
+    if (filter_size // 2 + origin < 0) or (filter_size // 2 + origin >=
+                                           filter_size):
         raise ValueError('invalid origin')
     mode = _ni_support._extend_mode_to_code(mode)
     _nd_image.generic_filter1d(input, function, filter_size, axis, output,
@@ -1092,9 +1136,9 @@ def generic_filter1d(input, function, filter_size, axis = -1,
 
 
 @docfiller
-def generic_filter(input, function, size = None, footprint = None,
-                   output = None, mode = "reflect", cval = 0.0, origin = 0,
-                   extra_arguments = (), extra_keywords = None):
+def generic_filter(input, function, size=None, footprint=None,
+                   output=None, mode="reflect", cval=0.0, origin=0,
+                   extra_arguments=(), extra_keywords = None):
     """Calculates a multi-dimensional filter using the given function.
 
     At each element the provided function is called. The input values
@@ -1105,7 +1149,7 @@ def generic_filter(input, function, size = None, footprint = None,
     ----------
     %(input)s
     function : callable
-        function to apply at each element
+        Function to apply at each element.
     %(size_foot)s
     %(output)s
     %(mode)s
