@@ -3,17 +3,20 @@ from __future__ import division, print_function, absolute_import
 import warnings
 from . import _minpack
 
-from numpy import atleast_1d, dot, take, triu, shape, eye, \
-                  transpose, zeros, product, greater, array, \
-                  all, where, isscalar, asarray, inf, abs, \
-                  finfo, inexact, issubdtype, dtype, sqrt
-from .optimize import Result, _check_unknown_options
+import numpy as np
+from numpy import (atleast_1d, dot, take, triu, shape, eye,
+                   transpose, zeros, product, greater, array,
+                   all, where, isscalar, asarray, inf, abs,
+                   finfo, inexact, issubdtype, dtype)
+from .optimize import OptimizeResult, _check_unknown_options, OptimizeWarning
 
 error = _minpack.error
 
 __all__ = ['fsolve', 'leastsq', 'fixed_point', 'curve_fit']
 
-def _check_func(checker, argname, thefunc, x0, args, numinputs, output_shape=None):
+
+def _check_func(checker, argname, thefunc, x0, args, numinputs,
+                output_shape=None):
     res = atleast_1d(thefunc(*((x0[:numinputs],) + args)))
     if (output_shape is not None) and (shape(res) != output_shape):
         if (output_shape[0] != 1):
@@ -93,15 +96,21 @@ def fsolve(func, x0, args=(), fprime=None, full_output=0,
     infodict : dict
         A dictionary of optional outputs with the keys:
 
-          * 'nfev' : number of function calls
-          * 'njev' : number of Jacobian calls
-          * 'fvec' : function evaluated at the output
-          * 'fjac' : the orthogonal matrix, q, produced by the QR
-                    factorization of the final approximate Jacobian
-                    matrix, stored column wise
-          * 'r' : upper triangular matrix produced by QR factorization
-                  of the same matrix
-          * 'qtf': the vector ``(transpose(q) * fvec)``
+        ``nfev``
+            number of function calls
+        ``njev``
+            number of Jacobian calls
+        ``fvec``
+            function evaluated at the output
+        ``fjac``
+            the orthogonal matrix, q, produced by the QR
+            factorization of the final approximate Jacobian
+            matrix, stored column wise
+        ``r``
+            upper triangular matrix produced by QR factorization
+            of the same matrix
+        ``qtf``
+            the vector ``(transpose(q) * fvec)``
 
     ier : int
         An integer flag.  Set to 1 if a solution was found, otherwise refer
@@ -109,10 +118,10 @@ def fsolve(func, x0, args=(), fprime=None, full_output=0,
     mesg : str
         If no solution is found, `mesg` details the cause of failure.
 
-    See also
+    See Also
     --------
     root : Interface to root finding algorithms for multivariate
-           functions. See the 'hybr' `method` in particular.
+    functions. See the 'hybr' `method` in particular.
 
     Notes
     -----
@@ -138,6 +147,7 @@ def fsolve(func, x0, args=(), fprime=None, full_output=0,
     else:
         return res['x']
 
+
 def _root_hybr(func, x0, args=(), jac=None,
                col_deriv=0, xtol=1.49012e-08, maxfev=0, band=None, eps=None,
                factor=100, diag=None, full_output=0, **unknown_options):
@@ -180,49 +190,49 @@ def _root_hybr(func, x0, args=(), jac=None,
     _check_unknown_options(unknown_options)
     epsfcn = eps
 
-    x0 = array(x0, ndmin=1)
+    x0 = asarray(x0).flatten()
     n = len(x0)
-    if type(args) != type(()):
+    if not isinstance(args, tuple):
         args = (args,)
     shape, dtype = _check_func('fsolve', 'func', func, x0, args, n, (n,))
     if epsfcn is None:
-        epsfcn = sqrt(finfo(dtype).eps)
+        epsfcn = finfo(dtype).eps
     Dfun = jac
     if Dfun is None:
         if band is None:
-            ml, mu = -10,-10
+            ml, mu = -10, -10
         else:
             ml, mu = band[:2]
-        if (maxfev == 0):
-            maxfev = 200*(n + 1)
+        if maxfev == 0:
+            maxfev = 200 * (n + 1)
         retval = _minpack._hybrd(func, x0, args, 1, xtol, maxfev,
                                  ml, mu, epsfcn, factor, diag)
     else:
-        _check_func('fsolve', 'fprime', Dfun, x0, args, n, (n,n))
+        _check_func('fsolve', 'fprime', Dfun, x0, args, n, (n, n))
         if (maxfev == 0):
-            maxfev = 100*(n + 1)
+            maxfev = 100 * (n + 1)
         retval = _minpack._hybrj(func, Dfun, x0, args, 1,
-                                 col_deriv, xtol, maxfev, factor,diag)
+                                 col_deriv, xtol, maxfev, factor, diag)
 
     x, status = retval[0], retval[-1]
 
-    errors = {0:["Improper input parameters were entered.",TypeError],
-              1:["The solution converged.", None],
-              2:["The number of calls to function has "
-                 "reached maxfev = %d." % maxfev, ValueError],
-              3:["xtol=%f is too small, no further improvement "
-                 "in the approximate\n  solution "
-                 "is possible." % xtol, ValueError],
-              4:["The iteration is not making good progress, as measured "
-                 "by the \n  improvement from the last five "
-                 "Jacobian evaluations.", ValueError],
-              5:["The iteration is not making good progress, "
-                 "as measured by the \n  improvement from the last "
-                 "ten iterations.", ValueError],
+    errors = {0: ["Improper input parameters were entered.", TypeError],
+              1: ["The solution converged.", None],
+              2: ["The number of calls to function has "
+                  "reached maxfev = %d." % maxfev, ValueError],
+              3: ["xtol=%f is too small, no further improvement "
+                  "in the approximate\n  solution "
+                  "is possible." % xtol, ValueError],
+              4: ["The iteration is not making good progress, as measured "
+                  "by the \n  improvement from the last five "
+                  "Jacobian evaluations.", ValueError],
+              5: ["The iteration is not making good progress, "
+                  "as measured by the \n  improvement from the last "
+                  "ten iterations.", ValueError],
               'unknown': ["An error occurred.", TypeError]}
 
     if status != 1 and not full_output:
-        if status in [2,3,4,5]:
+        if status in [2, 3, 4, 5]:
             msg = errors[status][0]
             warnings.warn(msg, RuntimeWarning)
         else:
@@ -233,7 +243,7 @@ def _root_hybr(func, x0, args=(), jac=None,
 
     info = retval[1]
     info['fun'] = info.pop('fvec')
-    sol = Result(x=x, success=(status==1), status=status)
+    sol = OptimizeResult(x=x, success=(status == 1), status=status)
     sol.update(info)
     try:
         sol['message'] = errors[status][0]
@@ -258,7 +268,8 @@ def leastsq(func, x0, args=(), Dfun=None, full_output=0,
     ----------
     func : callable
         should take at least one (possibly length N vector) argument and
-        returns M floating point numbers.
+        returns M floating point numbers. It must not return NaNs or
+        fitting might fail.
     x0 : ndarray
         The starting estimate for the minimization.
     args : tuple
@@ -282,10 +293,11 @@ def leastsq(func, x0, args=(), Dfun=None, full_output=0,
         The maximum number of calls to the function. If zero, then 100*(N+1) is
         the maximum where N is the number of elements in x0.
     epsfcn : float
-        A suitable step length for the forward-difference approximation of the
-        Jacobian (for Dfun=None). If epsfcn is less than the machine precision,
-        it is assumed that the relative errors in the functions are of the
-        order of the machine precision.
+        A variable used in determining a suitable step length for the forward-
+        difference approximation of the Jacobian (for Dfun=None). 
+        Normally the actual step length will be sqrt(epsfcn)*x
+        If epsfcn is less than the machine precision, it is assumed that the 
+        relative errors are of the order of the machine precision.
     factor : float
         A parameter determining the initial step bound
         (``factor * || diag * x||``). Should be in interval ``(0.1, 100)``.
@@ -299,28 +311,33 @@ def leastsq(func, x0, args=(), Dfun=None, full_output=0,
         call).
     cov_x : ndarray
         Uses the fjac and ipvt optional outputs to construct an
-        estimate of the jacobian around the solution.  ``None`` if a
+        estimate of the jacobian around the solution. None if a
         singular matrix encountered (indicates very flat curvature in
         some direction).  This matrix must be multiplied by the
         residual variance to get the covariance of the
         parameter estimates -- see curve_fit.
     infodict : dict
-        a dictionary of optional outputs with the key s::
+        a dictionary of optional outputs with the key s:
 
-            - 'nfev' : the number of function calls
-            - 'fvec' : the function evaluated at the output
-            - 'fjac' : A permutation of the R matrix of a QR
-                     factorization of the final approximate
-                     Jacobian matrix, stored column wise.
-                     Together with ipvt, the covariance of the
-                     estimate can be approximated.
-            - 'ipvt' : an integer array of length N which defines
-                     a permutation matrix, p, such that
-                     fjac*p = q*r, where r is upper triangular
-                     with diagonal elements of nonincreasing
-                     magnitude. Column j of p is column ipvt(j)
-                     of the identity matrix.
-            - 'qtf'  : the vector (transpose(q) * fvec).
+        ``nfev``
+            The number of function calls
+        ``fvec``
+            The function evaluated at the output
+        ``fjac``
+            A permutation of the R matrix of a QR
+            factorization of the final approximate
+            Jacobian matrix, stored column wise.
+            Together with ipvt, the covariance of the
+            estimate can be approximated.
+        ``ipvt``
+            An integer array of length N which defines
+            a permutation matrix, p, such that
+            fjac*p = q*r, where r is upper triangular
+            with diagonal elements of nonincreasing
+            magnitude. Column j of p is column ipvt(j)
+            of the identity matrix.
+        ``qtf``
+            The vector (transpose(q) * fvec).
 
     mesg : str
         A string message giving information about the cause of failure.
@@ -347,58 +364,60 @@ def leastsq(func, x0, args=(), Dfun=None, full_output=0,
          params
 
     """
-    x0 = array(x0, ndmin=1)
+    x0 = asarray(x0).flatten()
     n = len(x0)
-    if type(args) != type(()):
+    if not isinstance(args, tuple):
         args = (args,)
     shape, dtype = _check_func('leastsq', 'func', func, x0, args, n)
     m = shape[0]
     if n > m:
-        raise TypeError('Improper input: N=%s must not exceed M=%s' % (n,m))
+        raise TypeError('Improper input: N=%s must not exceed M=%s' % (n, m))
     if epsfcn is None:
-        epsfcn = sqrt(finfo(dtype).eps)
+        epsfcn = finfo(dtype).eps
     if Dfun is None:
         if maxfev == 0:
             maxfev = 200*(n + 1)
         retval = _minpack._lmdif(func, x0, args, full_output, ftol, xtol,
-                gtol, maxfev, epsfcn, factor, diag)
+                                 gtol, maxfev, epsfcn, factor, diag)
     else:
         if col_deriv:
-            _check_func('leastsq', 'Dfun', Dfun, x0, args, n, (n,m))
+            _check_func('leastsq', 'Dfun', Dfun, x0, args, n, (n, m))
         else:
-            _check_func('leastsq', 'Dfun', Dfun, x0, args, n, (m,n))
-        if (maxfev == 0):
-            maxfev = 100*(n + 1)
+            _check_func('leastsq', 'Dfun', Dfun, x0, args, n, (m, n))
+        if maxfev == 0:
+            maxfev = 100 * (n + 1)
         retval = _minpack._lmder(func, Dfun, x0, args, full_output, col_deriv,
-                ftol, xtol, gtol, maxfev, factor, diag)
+                                 ftol, xtol, gtol, maxfev, factor, diag)
 
-    errors = {0:["Improper input parameters.", TypeError],
-              1:["Both actual and predicted relative reductions "
-                 "in the sum of squares\n  are at most %f" % ftol, None],
-              2:["The relative error between two consecutive "
-                 "iterates is at most %f" % xtol, None],
-              3:["Both actual and predicted relative reductions in "
-                 "the sum of squares\n  are at most %f and the "
-                 "relative error between two consecutive "
-                 "iterates is at \n  most %f" % (ftol,xtol), None],
-              4:["The cosine of the angle between func(x) and any "
-                 "column of the\n  Jacobian is at most %f in "
-                 "absolute value" % gtol, None],
-              5:["Number of calls to function has reached "
-                 "maxfev = %d." % maxfev, ValueError],
-              6:["ftol=%f is too small, no further reduction "
-                 "in the sum of squares\n  is possible.""" % ftol, ValueError],
-              7:["xtol=%f is too small, no further improvement in "
-                 "the approximate\n  solution is possible." % xtol, ValueError],
-              8:["gtol=%f is too small, func(x) is orthogonal to the "
-                 "columns of\n  the Jacobian to machine "
-                 "precision." % gtol, ValueError],
-              'unknown':["Unknown error.", TypeError]}
+    errors = {0: ["Improper input parameters.", TypeError],
+              1: ["Both actual and predicted relative reductions "
+                  "in the sum of squares\n  are at most %f" % ftol, None],
+              2: ["The relative error between two consecutive "
+                  "iterates is at most %f" % xtol, None],
+              3: ["Both actual and predicted relative reductions in "
+                  "the sum of squares\n  are at most %f and the "
+                  "relative error between two consecutive "
+                  "iterates is at \n  most %f" % (ftol, xtol), None],
+              4: ["The cosine of the angle between func(x) and any "
+                  "column of the\n  Jacobian is at most %f in "
+                  "absolute value" % gtol, None],
+              5: ["Number of calls to function has reached "
+                  "maxfev = %d." % maxfev, ValueError],
+              6: ["ftol=%f is too small, no further reduction "
+                  "in the sum of squares\n  is possible.""" % ftol,
+                  ValueError],
+              7: ["xtol=%f is too small, no further improvement in "
+                  "the approximate\n  solution is possible." % xtol,
+                  ValueError],
+              8: ["gtol=%f is too small, func(x) is orthogonal to the "
+                  "columns of\n  the Jacobian to machine "
+                  "precision." % gtol, ValueError],
+              'unknown': ["Unknown error.", TypeError]}
 
     info = retval[-1]    # The FORTRAN return value
 
-    if (info not in [1,2,3,4] and not full_output):
-        if info in [5,6,7,8]:
+    if info not in [1, 2, 3, 4] and not full_output:
+        if info in [5, 6, 7, 8]:
             warnings.warn(errors[info][0], RuntimeWarning)
         else:
             try:
@@ -409,27 +428,31 @@ def leastsq(func, x0, args=(), Dfun=None, full_output=0,
     mesg = errors[info][0]
     if full_output:
         cov_x = None
-        if info in [1,2,3,4]:
+        if info in [1, 2, 3, 4]:
             from numpy.dual import inv
             from numpy.linalg import LinAlgError
-            perm = take(eye(n),retval[1]['ipvt']-1,0)
-            r = triu(transpose(retval[1]['fjac'])[:n,:])
+            perm = take(eye(n), retval[1]['ipvt'] - 1, 0)
+            r = triu(transpose(retval[1]['fjac'])[:n, :])
             R = dot(r, perm)
             try:
-                cov_x = inv(dot(transpose(R),R))
+                cov_x = inv(dot(transpose(R), R))
             except (LinAlgError, ValueError):
                 pass
         return (retval[0], cov_x) + retval[1:-1] + (mesg, info)
     else:
         return (retval[0], info)
 
+
 def _general_function(params, xdata, ydata, function):
     return function(xdata, *params) - ydata
 
+
 def _weighted_general_function(params, xdata, ydata, function, weights):
     return weights * (function(xdata, *params) - ydata)
 
-def curve_fit(f, xdata, ydata, p0=None, sigma=None, **kw):
+
+def curve_fit(f, xdata, ydata, p0=None, sigma=None, absolute_sigma=False,
+              check_finite=True, **kw):
     """
     Use non-linear least squares to fit a function, f, to data.
 
@@ -441,20 +464,37 @@ def curve_fit(f, xdata, ydata, p0=None, sigma=None, **kw):
         The model function, f(x, ...).  It must take the independent
         variable as the first argument and the parameters to fit as
         separate remaining arguments.
-    xdata : An N-length sequence or an (k,N)-shaped array
+    xdata : An M-length sequence or an (k,M)-shaped array
         for functions with k predictors.
         The independent variable where the data is measured.
-    ydata : N-length sequence
+    ydata : M-length sequence
         The dependent data --- nominally f(xdata, ...)
-    p0 : None, scalar, or M-length sequence
+    p0 : None, scalar, or N-length sequence
         Initial guess for the parameters.  If None, then the initial
         values will all be 1 (if the number of parameters for the function
         can be determined using introspection, otherwise a ValueError
         is raised).
-    sigma : None or N-length sequence
-        If not None, it represents the standard-deviation of ydata.
-        This vector, if given, will be used as weights in the
-        least-squares problem.
+    sigma : None or M-length sequence, optional
+        If not None, the uncertainties in the ydata array. These are used as
+        weights in the least-squares problem
+        i.e. minimising ``np.sum( ((f(xdata, *popt) - ydata) / sigma)**2 )``
+        If None, the uncertainties are assumed to be 1.
+    absolute_sigma : bool, optional
+        If False, `sigma` denotes relative weights of the data points.
+        The returned covariance matrix `pcov` is based on *estimated*
+        errors in the data, and is not affected by the overall
+        magnitude of the values in `sigma`. Only the relative
+        magnitudes of the `sigma` values matter.
+
+        If True, `sigma` describes one standard deviation errors of
+        the input data points. The estimated covariance in `pcov` is
+        based on these values.
+    check_finite : bool, optional
+        If True, check that the input arrays do not contain nans of infs,
+        and raise a ValueError if they do. Setting this parameter to
+        False may silently produce nonsensical results if the input arrays
+        do contain nans.
+        Default is True.
 
     Returns
     -------
@@ -462,8 +502,20 @@ def curve_fit(f, xdata, ydata, p0=None, sigma=None, **kw):
         Optimal values for the parameters so that the sum of the squared error
         of ``f(xdata, *popt) - ydata`` is minimized
     pcov : 2d array
-        The estimated covariance of popt.  The diagonals provide the variance
-        of the parameter estimate.
+        The estimated covariance of popt. The diagonals provide the variance
+        of the parameter estimate. To compute one standard deviation errors
+        on the parameters use ``perr = np.sqrt(np.diag(pcov))``.
+
+        How the `sigma` parameter affects the estimated covariance
+        depends on `absolute_sigma` argument, as described above.
+
+    Raises
+    ------
+    OptimizeWarning
+        if covariance of the parameters can not be estimated.
+
+    ValueError
+        if ydata and xdata contain NaNs.
 
     See Also
     --------
@@ -479,13 +531,13 @@ def curve_fit(f, xdata, ydata, p0=None, sigma=None, **kw):
     >>> import numpy as np
     >>> from scipy.optimize import curve_fit
     >>> def func(x, a, b, c):
-    ...     return a*np.exp(-b*x) + c
+    ...     return a * np.exp(-b * x) + c
 
-    >>> x = np.linspace(0,4,50)
-    >>> y = func(x, 2.5, 1.3, 0.5)
-    >>> yn = y + 0.2*np.random.normal(size=len(x))
+    >>> xdata = np.linspace(0, 4, 50)
+    >>> y = func(xdata, 2.5, 1.3, 0.5)
+    >>> ydata = y + 0.2 * np.random.normal(size=len(xdata))
 
-    >>> popt, pcov = curve_fit(func, x, yn)
+    >>> popt, pcov = curve_fit(func, xdata, ydata)
 
     """
     if p0 is None:
@@ -500,36 +552,63 @@ def curve_fit(f, xdata, ydata, p0=None, sigma=None, **kw):
         else:
             p0 = [1.0] * (len(args)-1)
 
+    # Check input arguments
     if isscalar(p0):
         p0 = array([p0])
 
+    # NaNs can not be handled
+    if check_finite:
+        ydata = np.asarray_chkfinite(ydata)
+    else:
+        ydata = np.asarray(ydata)
+    if isinstance(xdata, (list, tuple, np.ndarray)):
+        # `xdata` is passed straight to the user-defined `f`, so allow
+        # non-array_like `xdata`.
+        if check_finite:
+            xdata = np.asarray_chkfinite(xdata)
+        else:
+            xdata = np.asarray(xdata)
+
     args = (xdata, ydata, f)
     if sigma is None:
         func = _general_function
     else:
         func = _weighted_general_function
-        args += (1.0/asarray(sigma),)
+        args += (1.0 / asarray(sigma),)
 
     # Remove full_output from kw, otherwise we're passing it in twice.
     return_full = kw.pop('full_output', False)
     res = leastsq(func, p0, args=args, full_output=1, **kw)
     (popt, pcov, infodict, errmsg, ier) = res
 
-    if ier not in [1,2,3,4]:
+    if ier not in [1, 2, 3, 4]:
         msg = "Optimal parameters not found: " + errmsg
         raise RuntimeError(msg)
 
-    if (len(ydata) > len(p0)) and pcov is not None:
-        s_sq = (func(popt, *args)**2).sum()/(len(ydata)-len(p0))
-        pcov = pcov * s_sq
-    else:
-        pcov = inf
+    warn_cov = False
+    if pcov is None:
+        # indeterminate covariance
+        pcov = zeros((len(popt), len(popt)), dtype=float)
+        pcov.fill(inf)
+        warn_cov = True
+    elif not absolute_sigma:
+        if len(ydata) > len(p0):
+            s_sq = (asarray(func(popt, *args))**2).sum() / (len(ydata) - len(p0))
+            pcov = pcov * s_sq
+        else:
+            pcov.fill(inf)
+            warn_cov = True
+
+    if warn_cov:
+        warnings.warn('Covariance of the parameters could not be estimated',
+                category=OptimizeWarning)
 
     if return_full:
         return popt, pcov, infodict, errmsg, ier
     else:
         return popt, pcov
 
+
 def check_gradient(fcn, Dfcn, x0, args=(), col_deriv=0):
     """Perform a simple check on the gradient for correctness.
 
@@ -538,12 +617,12 @@ def check_gradient(fcn, Dfcn, x0, args=(), col_deriv=0):
     x = atleast_1d(x0)
     n = len(x)
     x = x.reshape((n,))
-    fvec = atleast_1d(fcn(x,*args))
+    fvec = atleast_1d(fcn(x, *args))
     m = len(fvec)
     fvec = fvec.reshape((m,))
     ldfjac = m
-    fjac = atleast_1d(Dfcn(x,*args))
-    fjac = fjac.reshape((m,n))
+    fjac = atleast_1d(Dfcn(x, *args))
+    fjac = fjac.reshape((m, n))
     if col_deriv == 0:
         fjac = transpose(fjac)
 
@@ -552,7 +631,7 @@ def check_gradient(fcn, Dfcn, x0, args=(), col_deriv=0):
     fvecp = None
     _minpack._chkder(m, n, x, fvec, fjac, ldfjac, xp, fvecp, 1, err)
 
-    fvecp = atleast_1d(fcn(xp,*args))
+    fvecp = atleast_1d(fcn(xp, *args))
     fvecp = fvecp.reshape((m,))
     _minpack._chkder(m, n, x, fvec, fjac, ldfjac, xp, fvecp, 2, err)
 
@@ -561,7 +640,6 @@ def check_gradient(fcn, Dfcn, x0, args=(), col_deriv=0):
     return (good, err)
 
 
-# Steffensen's Method using Aitken's Del^2 convergence acceleration.
 def fixed_point(func, x0, args=(), xtol=1e-8, maxiter=500):
     """
     Find a fixed point of the function.
