@@ -11,11 +11,37 @@ To run it in its simplest form::
 """
 from __future__ import division, print_function, absolute_import
 
-from numpy.testing import assert_raises, assert_allclose, \
-        assert_equal, assert_, TestCase, run_module_suite, dec
+import warnings
+
+import numpy as np
+from numpy.testing import (assert_raises, assert_allclose, assert_equal,
+                           assert_, TestCase, run_module_suite, dec,
+                           assert_almost_equal)
 
 from scipy import optimize
-import numpy as np
+
+
+def test_check_grad():
+    # Verify if check_grad is able to estimate the derivative of the
+    # logistic function.
+
+    def logit(x):
+        return 1 / (1 + np.exp(-x))
+
+    def der_logit(x):
+        return np.exp(-x) / (1 + np.exp(-x))**2
+
+    x0 = np.array([1.5])
+
+    r = optimize.check_grad(logit, der_logit, x0)
+    assert_almost_equal(r, 0)
+
+    r = optimize.check_grad(logit, der_logit, x0, epsilon=1e-6)
+    assert_almost_equal(r, 0)
+
+    # Check if the epsilon parameter is being considered.
+    r = abs(optimize.check_grad(logit, der_logit, x0, epsilon=1e-1) - 0)
+    assert_(r > 1e-7)
 
 
 class TestOptimize(object):
@@ -130,12 +156,9 @@ class TestOptimize(object):
         func = lambda x: x
         fprime = lambda x: np.ones_like(x)
         x0 = [np.nan]
-        olderr = np.seterr(over='ignore')
-        try:
+        with np.errstate(over='ignore', invalid='ignore'):
             x = optimize.fmin_bfgs(func, x0, fprime, disp=False)
             assert_(np.isnan(func(x)))
-        finally:
-            np.seterr(**olderr)
 
     def test_bfgs_numerical_jacobian(self):
         """ BFGS with numerical jacobian and a vector epsilon parameter """
@@ -166,6 +189,15 @@ class TestOptimize(object):
         finally:
             np.seterr(**olderr)
 
+    def test_bfgs_gh_2169(self):
+        def f(x):
+            if x < 0:
+                return 1.79769313e+308
+            else:
+                return x + 1./x
+        xs = optimize.fmin_bfgs(f, [10.], disp=False)
+        assert_allclose(xs, 1.0, rtol=1e-4, atol=1e-4)
+
     def test_powell(self, use_wrapper=False):
         """ Powell (direction set) optimization routine
         """
@@ -392,11 +424,18 @@ class TestOptimize(object):
     def test_minimize_l_bfgs_b(self):
         """ Minimize with L-BFGS-B method """
         opts = {'disp': False, 'maxiter': self.maxiter}
-        x = optimize.minimize(self.func, self.startparams,
+        r = optimize.minimize(self.func, self.startparams,
                               method='L-BFGS-B', jac=self.grad,
-                              options=opts)['x']
-        assert_allclose(self.func(x), self.func(self.solution),
+                              options=opts)
+        assert_allclose(self.func(r.x), self.func(self.solution),
                         atol=1e-6)
+        # approximate jacobian
+        ra = optimize.minimize(self.func, self.startparams,
+                               method='L-BFGS-B', options=opts)
+        assert_allclose(self.func(ra.x), self.func(self.solution),
+                        atol=1e-6)
+        # check that function evaluations in approximate jacobian are counted
+        assert_(ra.nfev > r.nfev)
 
     def test_minimize_l_bfgs_b_ftol(self):
         # Check that the `ftol` parameter in l_bfgs_b works as expected
@@ -415,6 +454,44 @@ class TestOptimize(object):
 
             assert_allclose(v, self.func(self.solution), rtol=tol)
 
+    def test_custom(self):
+        # This function comes from the documentation example.
+        def custmin(fun, x0, args=(), maxfev=None, stepsize=0.1,
+                maxiter=100, callback=None, **options):
+            bestx = x0
+            besty = fun(x0)
+            funcalls = 1
+            niter = 0
+            improved = True
+            stop = False
+
+            while improved and not stop and niter < maxiter:
+                improved = False
+                niter += 1
+                for dim in range(np.size(x0)):
+                    for s in [bestx[dim] - stepsize, bestx[dim] + stepsize]:
+                        testx = np.copy(bestx)
+                        testx[dim] = s
+                        testy = fun(testx, *args)
+                        funcalls += 1
+                        if testy < besty:
+                            besty = testy
+                            bestx = testx
+                            improved = True
+                    if callback is not None:
+                        callback(bestx)
+                    if maxfev is not None and funcalls >= maxfev:
+                        stop = True
+                        break
+
+            return optimize.OptimizeResult(fun=besty, x=bestx, nit=niter,
+                                           nfev=funcalls, success=(niter > 1))
+
+        x0 = [1.35, 0.9, 0.8, 1.1, 1.2]
+        res = optimize.minimize(optimize.rosen, x0, method=custmin,
+                                options=dict(stepsize=0.05))
+        assert_allclose(res.x, 1.0, rtol=1e-4, atol=1e-4)
+
     def test_minimize(self):
         """Tests for the minimize wrapper."""
         self.setUp()
@@ -433,6 +510,8 @@ class TestOptimize(object):
         self.test_neldermead(True)
         self.setUp()
         self.test_powell(True)
+        self.setUp()
+        self.test_custom()
 
     def test_minimize_tol_parameter(self):
         # Check that the minimize() tol= argument does something
@@ -451,12 +530,16 @@ class TestOptimize(object):
                 jac = None
             else:
                 jac = dfunc
-            sol1 = optimize.minimize(func, [1,1], jac=jac, tol=1e-10,
-                                     method=method)
-            sol2 = optimize.minimize(func, [1,1], jac=jac, tol=1.0,
-                                     method=method)
-            assert_(func(sol1.x) < func(sol2.x),
-                    "%s: %s vs. %s" % (method, func(sol1.x), func(sol2.x)))
+
+            with warnings.catch_warnings():
+                # suppress deprecation warning for 'anneal'
+                warnings.filterwarnings('ignore', category=DeprecationWarning)
+                sol1 = optimize.minimize(func, [1,1], jac=jac, tol=1e-10,
+                                         method=method)
+                sol2 = optimize.minimize(func, [1,1], jac=jac, tol=1.0,
+                                         method=method)
+                assert_(func(sol1.x) < func(sol2.x),
+                        "%s: %s vs. %s" % (method, func(sol1.x), func(sol2.x)))
 
     def test_no_increase(self):
         # Check that the solver doesn't return a value worse than the
@@ -484,10 +567,61 @@ class TestOptimize(object):
             assert_(func(sol.x) <= f0)
 
         for method in ['nelder-mead', 'powell', 'cg', 'bfgs',
-                       'newton-cg', 'anneal', 'l-bfgs-b', 'tnc',
+                       'newton-cg', 'l-bfgs-b', 'tnc',
                        'cobyla', 'slsqp']:
             yield check, method
 
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore', category=DeprecationWarning)
+            yield check, 'anneal'
+
+    def test_slsqp_respect_bounds(self):
+        # github issue 3108
+        def f(x):
+            return sum((x - np.array([1., 2., 3., 4.]))**2)
+
+        def cons(x):
+            a = np.array([[-1, -1, -1, -1], [-3, -3, -2, -1]])
+            return np.concatenate([np.dot(a, x) + np.array([5, 10]), x])
+
+        x0 = np.array([0.5, 1., 1.5, 2.])
+        res = optimize.minimize(f, x0, method='slsqp',
+                                constraints={'type': 'ineq', 'fun': cons})
+        assert_allclose(res.x, np.array([0., 2, 5, 8])/3, atol=1e-12)
+
+    def test_minimize_automethod(self):
+        def f(x):
+            return x**2
+
+        def cons(x):
+            return x - 2
+
+        x0 = np.array([10.])
+        sol_0 = optimize.minimize(f, x0)
+        sol_1 = optimize.minimize(f, x0, constraints=[{'type': 'ineq', 'fun': cons}])
+        sol_2 = optimize.minimize(f, x0, bounds=[(5, 10)])
+        sol_3 = optimize.minimize(f, x0, constraints=[{'type': 'ineq', 'fun': cons}], bounds=[(5, 10)])
+        sol_4 = optimize.minimize(f, x0, constraints=[{'type': 'ineq', 'fun': cons}], bounds=[(1, 10)])
+        for sol in [sol_0, sol_1, sol_2, sol_3, sol_4]:
+            assert_(sol.success)
+        assert_allclose(sol_0.x, 0, atol=1e-8)
+        assert_allclose(sol_1.x, 2, atol=1e-8)
+        assert_allclose(sol_2.x, 5, atol=1e-8)
+        assert_allclose(sol_3.x, 5, atol=1e-8)
+        assert_allclose(sol_4.x, 2, atol=1e-8)
+
+    def test_minimize_coerce_args_param(self):
+        # github issue #3503
+        def Y(x, c):
+            return np.sum((x-c)**2)
+
+        def dY_dx(x, c=None):
+            return 2*(x-c)
+
+        c = np.array([3, 1, 4, 1, 5, 9, 2, 6, 5, 3, 5])
+        xinit = np.random.randn(len(c))
+        optimize.minimize(Y, xinit, jac=dY_dx, args=(c), method="BFGS")
+
 
 class TestLBFGSBBounds(TestCase):
     """ Tests for L-BFGS-B with bounds """
@@ -577,8 +711,11 @@ class TestOptimizeScalar(TestCase):
         assert_raises(ValueError, optimize.fminbound, self.fun, 5, 1)
 
     def test_fminbound_scalar(self):
-        assert_raises(ValueError, optimize.fminbound, self.fun,
-                      np.zeros(2), 1)
+        try:
+            optimize.fminbound(self.fun, np.zeros((1, 2)), 1)
+            self.fail("exception not raised")
+        except ValueError as e:
+            assert_('must be scalar' in str(e))
 
         x = optimize.fminbound(self.fun, 1, np.array(5))
         assert_allclose(x, self.solution, atol=1e-6)
@@ -636,6 +773,44 @@ class TestOptimizeScalar(TestCase):
                                      method='bounded').x
         assert_allclose(x, self.solution, atol=1e-6)
 
+    def test_minimize_scalar_custom(self):
+        # This function comes from the documentation example.
+        def custmin(fun, bracket, args=(), maxfev=None, stepsize=0.1,
+                maxiter=100, callback=None, **options):
+            bestx = (bracket[1] + bracket[0]) / 2.0
+            besty = fun(bestx)
+            funcalls = 1
+            niter = 0
+            improved = True
+            stop = False
+
+            while improved and not stop and niter < maxiter:
+                improved = False
+                niter += 1
+                for testx in [bestx - stepsize, bestx + stepsize]:
+                    testy = fun(testx, *args)
+                    funcalls += 1
+                    if testy < besty:
+                        besty = testy
+                        bestx = testx
+                        improved = True
+                if callback is not None:
+                    callback(bestx)
+                if maxfev is not None and funcalls >= maxfev:
+                    stop = True
+                    break
+
+            return optimize.OptimizeResult(fun=besty, x=bestx, nit=niter,
+                                           nfev=funcalls, success=(niter > 1))
+
+        res = optimize.minimize_scalar(self.fun, bracket=(0, 4), method=custmin,
+                                       options=dict(stepsize=0.05))
+        assert_allclose(res.x, self.solution, atol=1e-6)
+
+    def test_minimize_scalar_coerce_args_param(self):
+        # github issue #3503
+        optimize.minimize_scalar(self.fun, args=1.5)
+
 
 class TestNewtonCg(object):
     def test_rosenbrock(self):
@@ -698,5 +873,27 @@ himmelblau_x0 = [-0.27, -0.9]
 himmelblau_xopt = [3, 2]
 himmelblau_min = 0.0
 
+
+def test_minimize_multiple_constraints():
+    # Regression test for gh-4240.
+    def func(x):
+        return np.array([25 - 0.2 * x[0] - 0.4 * x[1] - 0.33 * x[2]])
+
+    def func1(x):
+        return np.array([x[1]])
+
+    def func2(x):
+        return np.array([x[2]])
+
+    cons = ({'type': 'ineq', 'fun': func},
+            {'type': 'ineq', 'fun': func1},
+            {'type': 'ineq', 'fun': func2})
+
+    f = lambda x: -1 * (x[0] + x[1] + x[2])
+
+    res = optimize.minimize(f, [0, 0, 0], method='SLSQP', constraints=cons)
+    assert_allclose(res.x, [125, 0, 0], atol=1e-10)
+
+
 if __name__ == "__main__":
     run_module_suite()
