@@ -118,7 +118,7 @@ def _restore(M, shape):
 
 
 def abcd_normalize(A=None, B=None, C=None, D=None):
-    """Check state-space matrices and ensure they are rank-2.
+    """Check state-space matrices and ensure they are two-dimensional.
 
     If enough information on the system is provided, that is, enough
     properly-shaped arrays are passed to the function, the missing ones
@@ -194,11 +194,8 @@ def ss2tf(A, B, C, D, input=0):
         raise ValueError("System does not have the input specified.")
 
     # make MOSI from possibly MOMI system.
-    if B.shape[-1] != 0:
-        B = B[:, input]
-    B.shape = (B.shape[0], 1)
-    if D.shape[-1] != 0:
-        D = D[:, input]
+    B = B[:, input:input + 1]
+    D = D[:, input:input + 1]
 
     try:
         den = poly(A)
@@ -431,20 +428,35 @@ class lti(object):
                                                           self.C, self.D)
 
     def impulse(self, X0=None, T=None, N=None):
+        """
+        Return the impulse response of a continuous-time system.
+        See `scipy.signal.impulse` for details.
+        """
         return impulse(self, X0=X0, T=T, N=N)
 
     def step(self, X0=None, T=None, N=None):
+        """
+        Return the step response of a continuous-time system.
+        See `scipy.signal.step` for details.
+        """
         return step(self, X0=X0, T=T, N=N)
 
     def output(self, U, T, X0=None):
+        """
+        Return the response of a continuous-time system to input `U`.
+        See `scipy.signal.lsim` for details.
+        """
         return lsim(self, U, T, X0=X0)
 
     def bode(self, w=None, n=100):
         """
-        Calculate Bode magnitude and phase data.
+        Calculate Bode magnitude and phase data of a continuous-time system.
 
         Returns a 3-tuple containing arrays of frequencies [rad/s], magnitude
-        [dB] and phase [deg]. See scipy.signal.bode for details.
+        [dB] and phase [deg]. See `scipy.signal.bode` for details.
+
+        Notes
+        -----
 
         .. versionadded:: 0.11.0
 
@@ -466,11 +478,12 @@ class lti(object):
         return bode(self, w=w, n=n)
 
     def freqresp(self, w=None, n=10000):
-        """Calculate the frequency response of a continuous-time system.
+        """
+        Calculate the frequency response of a continuous-time system.
 
         Returns a 2-tuple containing arrays of frequencies [rad/s] and
         complex magnitude.
-        See scipy.signal.freqresp for details.
+        See `scipy.signal.freqresp` for details.
 
         """
         return freqresp(self, w=w, n=n)
@@ -972,8 +985,6 @@ def bode(system, w=None, n=100):
     """
     Calculate Bode magnitude and phase data of a continuous-time system.
 
-    .. versionadded:: 0.11.0
-
     Parameters
     ----------
     system : an instance of the LTI class or a tuple describing the system.
@@ -1002,6 +1013,11 @@ def bode(system, w=None, n=100):
     phase : 1D ndarray
         Phase array [deg]
 
+    Notes
+    -----
+
+    .. versionadded:: 0.11.0
+
     Examples
     --------
     >>> from scipy import signal
