@@ -14,11 +14,13 @@ from numpy import (isscalar, r_, log, sum, around, unique, asarray,
 from numpy.testing.decorators import setastest
 
 from scipy.lib.six import string_types
+from scipy.lib._numpy_compat import count_nonzero
 from scipy import optimize
 from scipy import special
 from . import statlib
 from . import stats
 from .stats import find_repeats
+from .contingency import chi2_contingency
 from . import distributions
 from ._distn_infrastructure import rv_generic
 
@@ -27,7 +29,7 @@ __all__ = ['mvsdist',
            'bayes_mvs', 'kstat', 'kstatvar', 'probplot', 'ppcc_max', 'ppcc_plot',
            'boxcox_llf', 'boxcox', 'boxcox_normmax', 'boxcox_normplot',
            'shapiro', 'anderson', 'ansari', 'bartlett', 'levene', 'binom_test',
-           'fligner', 'mood', 'wilcoxon',
+           'fligner', 'mood', 'wilcoxon', 'median_test',
            'pdf_fromgamma', 'circmean', 'circvar', 'circstd', 'anderson_ksamp'
            ]
 
@@ -445,7 +447,7 @@ def probplot(x, sparams=(), dist='norm', fit=True, plot=None):
         ymax = amax(x)
         posx = xmin + 0.70 * (xmax - xmin)
         posy = ymin + 0.01 * (ymax - ymin)
-        plot.text(posx, posy, "$R^2=%1.4f$" % r)
+        plot.text(posx, posy, "$R^2=%1.4f$" % r**2)
 
     if fit:
         return (osm, osr), (slope, intercept, r)
@@ -831,7 +833,7 @@ def boxcox_normmax(x, brack=(-2.0, 2.0), method='pearsonr'):
     methods = {'pearsonr': _pearsonr,
                'mle': _mle,
                'all': _all}
-    if not method in methods.keys():
+    if method not in methods.keys():
         raise ValueError("Method %s not recognized." % method)
 
     optimfunc = methods[method]
@@ -989,7 +991,7 @@ def shapiro(x, a=None, reta=False):
         init = 1
     y = sort(x)
     a, w, pw, ifault = statlib.swilk(y, a[:N//2], init)
-    if not ifault in [0,2]:
+    if ifault not in [0,2]:
         warnings.warn(str(ifault))
     if N > 5000:
         warnings.warn("p-value may not be accurate for N > 5000.")
@@ -1078,7 +1080,7 @@ def anderson(x,dist='norm'):
            pp. 591-595.
 
     """
-    if not dist in ['norm','expon','gumbel','extreme1','logistic']:
+    if dist not in ['norm','expon','gumbel','extreme1','logistic']:
         raise ValueError("Invalid distribution; dist must be 'norm', "
                             "'expon', 'gumbel', 'extreme1' or 'logistic'.")
     y = sort(x)
@@ -1111,14 +1113,14 @@ def anderson(x,dist='norm'):
         critical = around(_Avals_logistic / (1.0+0.25/N),3)
     else:  # (dist == 'gumbel') or (dist == 'extreme1'):
         # the following is incorrect, see ticket:1097
-##        def fixedsolve(th,xj,N):
-##            val = stats.sum(xj)*1.0/N
-##            tmp = exp(-xj/th)
-##            term = sum(xj*tmp,axis=0)
-##            term /= sum(tmp,axis=0)
-##            return val - term
-##        s = optimize.fixed_point(fixedsolve, 1.0, args=(x,N),xtol=1e-5)
-##        xbar = -s*log(sum(exp(-x/s),axis=0)*1.0/N)
+        #def fixedsolve(th,xj,N):
+        #    val = stats.sum(xj)*1.0/N
+        #    tmp = exp(-xj/th)
+        #    term = sum(xj*tmp,axis=0)
+        #    term /= sum(tmp,axis=0)
+        #    return val - term
+        #s = optimize.fixed_point(fixedsolve, 1.0, args=(x,N),xtol=1e-5)
+        #xbar = -s*log(sum(exp(-x/s),axis=0)*1.0/N)
         xbar, s = distributions.gumbel_l.fit(x)
         w = (y-xbar)/s
         z = distributions.gumbel_l.cdf(w)
@@ -1274,8 +1276,8 @@ def anderson_ksamp(samples, midrank=True):
            Anderson-Darling Tests, Journal of the American Statistical
            Association, Vol. 82, pp. 918-924.
 
-    Examples:
-    ---------
+    Examples
+    --------
     >>> from scipy import stats
     >>> np.random.seed(314159)
 
@@ -1301,8 +1303,8 @@ def anderson_ksamp(samples, midrank=True):
     (-0.73091722665244196,
       array([ 0.44925884,  1.3052767 ,  1.9434184 ,  2.57696569,  3.41634856]),
       0.8789283903979661)
-    """
 
+    """
     k = len(samples)
     if (k < 2):
         raise ValueError("anderson_ksamp needs at least two samples")
@@ -1452,7 +1454,7 @@ def bartlett(*args):
     Bartlett's test tests the null hypothesis that all input samples
     are from populations with equal variances.  For samples
     from significantly non-normal populations, Levene's test
-    `levene`_ is more robust.
+    `levene` is more robust.
 
     Parameters
     ----------
@@ -1491,7 +1493,7 @@ def bartlett(*args):
     return T, pval
 
 
-def levene(*args,**kwds):
+def levene(*args, **kwds):
     """
     Perform Levene test for equal variances.
 
@@ -1543,7 +1545,8 @@ def levene(*args,**kwds):
     proportiontocut = 0.05
     for kw, value in kwds.items():
         if kw not in ['center', 'proportiontocut']:
-            raise TypeError("levene() got an unexpected keyword argument '%s'" % kw)
+            raise TypeError("levene() got an unexpected keyword "
+                            "argument '%s'" % kw)
         if kw == 'center':
             center = value
         else:
@@ -1553,9 +1556,9 @@ def levene(*args,**kwds):
     if k < 2:
         raise ValueError("Must enter at least two input sample vectors.")
     Ni = zeros(k)
-    Yci = zeros(k,'d')
+    Yci = zeros(k, 'd')
 
-    if not center in ['mean','median','trimmed']:
+    if center not in ['mean', 'median', 'trimmed']:
         raise ValueError("Keyword argument <center> must be 'mean', 'median'"
               + "or 'trimmed'.")
 
@@ -1564,42 +1567,43 @@ def levene(*args,**kwds):
     elif center == 'mean':
         func = lambda x: np.mean(x, axis=0)
     else:  # center == 'trimmed'
-        args = tuple(stats.trimboth(np.sort(arg), proportiontocut) for arg in args)
+        args = tuple(stats.trimboth(np.sort(arg), proportiontocut)
+                     for arg in args)
         func = lambda x: np.mean(x, axis=0)
 
     for j in range(k):
         Ni[j] = len(args[j])
         Yci[j] = func(args[j])
-    Ntot = sum(Ni,axis=0)
+    Ntot = sum(Ni, axis=0)
 
     # compute Zij's
     Zij = [None]*k
     for i in range(k):
         Zij[i] = abs(asarray(args[i])-Yci[i])
     # compute Zbari
-    Zbari = zeros(k,'d')
+    Zbari = zeros(k, 'd')
     Zbar = 0.0
     for i in range(k):
         Zbari[i] = np.mean(Zij[i], axis=0)
         Zbar += Zbari[i]*Ni[i]
     Zbar /= Ntot
 
-    numer = (Ntot-k)*sum(Ni*(Zbari-Zbar)**2,axis=0)
+    numer = (Ntot-k) * sum(Ni*(Zbari-Zbar)**2, axis=0)
 
     # compute denom_variance
     dvar = 0.0
     for i in range(k):
-        dvar += sum((Zij[i]-Zbari[i])**2,axis=0)
+        dvar += sum((Zij[i]-Zbari[i])**2, axis=0)
 
     denom = (k-1.0)*dvar
 
     W = numer / denom
-    pval = distributions.f.sf(W,k-1,Ntot-k)  # 1 - cdf
+    pval = distributions.f.sf(W, k-1, Ntot-k)  # 1 - cdf
     return W, pval
 
 
 @setastest(False)
-def binom_test(x,n=None,p=0.5):
+def binom_test(x, n=None, p=0.5):
     """
     Perform a test that the probability of success is p.
 
@@ -1644,35 +1648,37 @@ def binom_test(x,n=None,p=0.5):
     if (p > 1.0) or (p < 0.0):
         raise ValueError("p must be in range [0,1]")
 
-    d = distributions.binom.pmf(x,n,p)
+    d = distributions.binom.pmf(x, n, p)
     rerr = 1+1e-7
     if (x == p*n):
         # special case as shortcut, would also be handled by `else` below
         pval = 1.
     elif (x < p*n):
-        i = np.arange(np.ceil(p*n),n+1)
-        y = np.sum(distributions.binom.pmf(i,n,p) <= d*rerr,axis=0)
-        pval = distributions.binom.cdf(x,n,p) + distributions.binom.sf(n-y,n,p)
+        i = np.arange(np.ceil(p*n), n+1)
+        y = np.sum(distributions.binom.pmf(i, n, p) <= d*rerr, axis=0)
+        pval = (distributions.binom.cdf(x, n, p) +
+                distributions.binom.sf(n-y, n, p))
     else:
         i = np.arange(np.floor(p*n) + 1)
-        y = np.sum(distributions.binom.pmf(i,n,p) <= d*rerr,axis=0)
-        pval = distributions.binom.cdf(y-1,n,p) + distributions.binom.sf(x-1,n,p)
+        y = np.sum(distributions.binom.pmf(i, n, p) <= d*rerr, axis=0)
+        pval = (distributions.binom.cdf(y-1, n, p) +
+                distributions.binom.sf(x-1, n, p))
 
-    return min(1.0,pval)
+    return min(1.0, pval)
 
 
-def _apply_func(x,g,func):
+def _apply_func(x, g, func):
     # g is list of indices into x
     #  separating x into different groups
     #  func should be applied over the groups
-    g = unique(r_[0,g,len(x)])
+    g = unique(r_[0, g, len(x)])
     output = []
     for k in range(len(g)-1):
         output.append(func(x[g[k]:g[k+1]]))
     return asarray(output)
 
 
-def fligner(*args,**kwds):
+def fligner(*args, **kwds):
     """
     Perform Fligner's test for equal variances.
 
@@ -1684,11 +1690,10 @@ def fligner(*args,**kwds):
     Parameters
     ----------
     sample1, sample2, ... : array_like
-        arrays of sample data.  Need not be the same length
+        Arrays of sample data.  Need not be the same length.
     center : {'mean', 'median', 'trimmed'}, optional
-        keyword argument controlling which function of the data
-        is used in computing the test statistic.  The default
-        is 'median'.
+        Keyword argument controlling which function of the data is used in
+        computing the test statistic.  The default is 'median'.
     proportiontocut : float, optional
         When `center` is 'trimmed', this gives the proportion of data points
         to cut from each end. (See `scipy.stats.trim_mean`.)
@@ -1697,15 +1702,15 @@ def fligner(*args,**kwds):
     Returns
     -------
     Xsq : float
-        the test statistic
+        The test statistic.
     p-value : float
-        the p-value for the hypothesis test
+        The p-value for the hypothesis test.
 
     Notes
     -----
-    As with Levene's test there are three variants
-    of Fligner's test that differ by the measure of central
-    tendency used in the test.  See `levene` for more information.
+    As with Levene's test there are three variants of Fligner's test that
+    differ by the measure of central tendency used in the test.  See `levene`
+    for more information.
 
     References
     ----------
@@ -1721,7 +1726,8 @@ def fligner(*args,**kwds):
     proportiontocut = 0.05
     for kw, value in kwds.items():
         if kw not in ['center', 'proportiontocut']:
-            raise TypeError("fligner() got an unexpected keyword argument '%s'" % kw)
+            raise TypeError("fligner() got an unexpected keyword "
+                            "argument '%s'" % kw)
         if kw == 'center':
             center = value
         else:
@@ -1731,7 +1737,7 @@ def fligner(*args,**kwds):
     if k < 2:
         raise ValueError("Must enter at least two input sample vectors.")
 
-    if not center in ['mean','median','trimmed']:
+    if center not in ['mean','median','trimmed']:
         raise ValueError("Keyword argument <center> must be 'mean', 'median'"
               + "or 'trimmed'.")
 
@@ -1745,9 +1751,9 @@ def fligner(*args,**kwds):
 
     Ni = asarray([len(args[j]) for j in range(k)])
     Yci = asarray([func(args[j]) for j in range(k)])
-    Ntot = sum(Ni,axis=0)
+    Ntot = sum(Ni, axis=0)
     # compute Zij's
-    Zij = [abs(asarray(args[i])-Yci[i]) for i in range(k)]
+    Zij = [abs(asarray(args[i]) - Yci[i]) for i in range(k)]
     allZij = []
     g = [0]
     for i in range(k):
@@ -1755,14 +1761,14 @@ def fligner(*args,**kwds):
         g.append(len(allZij))
 
     ranks = stats.rankdata(allZij)
-    a = distributions.norm.ppf(ranks/(2*(Ntot+1.0)) + 0.5)
+    a = distributions.norm.ppf(ranks/(2*(Ntot + 1.0)) + 0.5)
 
     # compute Aibar
-    Aibar = _apply_func(a,g,sum) / Ni
+    Aibar = _apply_func(a, g, sum) / Ni
     anbar = np.mean(a, axis=0)
-    varsq = np.var(a,axis=0, ddof=1)
-    Xsq = sum(Ni*(asarray(Aibar)-anbar)**2.0,axis=0)/varsq
-    pval = distributions.chi2.sf(Xsq,k-1)  # 1 - cdf
+    varsq = np.var(a, axis=0, ddof=1)
+    Xsq = sum(Ni*(asarray(Aibar) - anbar)**2.0, axis=0)/varsq
+    pval = distributions.chi2.sf(Xsq, k - 1)  # 1 - cdf
     return Xsq, pval
 
 
@@ -1788,7 +1794,7 @@ def mood(x, y, axis=0):
     -------
     z : scalar or ndarray
         The z-score for the hypothesis test.  For 1-D inputs a scalar is
-        returned;
+        returned.
     p-value : scalar ndarray
         The p-value for the hypothesis test.
 
@@ -1941,7 +1947,7 @@ def wilcoxon(x, y=None, zero_method="wilcox", correction=False):
 
     """
 
-    if not zero_method in ["wilcox", "pratt", "zsplit"]:
+    if zero_method not in ["wilcox", "pratt", "zsplit"]:
         raise ValueError("Zero method should be either 'wilcox' \
                           or 'pratt' or 'zsplit'")
 
@@ -1987,6 +1993,211 @@ def wilcoxon(x, y=None, zero_method="wilcox", correction=False):
     return T, prob
 
 
+@setastest(False)
+def median_test(*args, **kwds):
+    """
+    Mood's median test.
+
+    Test that two or more samples come from populations with the same median.
+
+    Let ``n = len(args)`` be the number of samples.  The "grand median" of
+    all the data is computed, and a contingency table is formed by
+    classifying the values in each sample as being above or below the grand
+    median.  The contingency table, along with `correction` and `lambda_`,
+    are passed to `scipy.stats.chi2_contingency` to compute the test statistic
+    and p-value.
+
+    Parameters
+    ----------
+    sample1, sample2, ... : array_like
+        The set of samples.  There must be at least two samples.
+        Each sample must be a one-dimensional sequence containing at least
+        one value.  The samples are not required to have the same length.
+    ties : str, optional
+        Determines how values equal to the grand median are classified in
+        the contingency table.  The string must be one of::
+
+            "below":
+                Values equal to the grand median are counted as "below".
+            "above":
+                Values equal to the grand median are counted as "above".
+            "ignore":
+                Values equal to the grand median are not counted.
+
+        The default is "below".
+    correction : bool, optional
+        If True, *and* there are just two samples, apply Yates' correction
+        for continuity when computing the test statistic associated with
+        the contingency table.  Default is True.
+    lambda_ : float or str, optional.
+        By default, the statistic computed in this test is Pearson's
+        chi-squared statistic.  `lambda_` allows a statistic from the
+        Cressie-Read power divergence family to be used instead.  See
+        `power_divergence` for details.
+        Default is 1 (Pearson's chi-squared statistic).
+
+    Returns
+    -------
+    stat : float
+        The test statistic.  The statistic that is returned is determined by
+        `lambda_`.  The default is Pearson's chi-squared statistic.
+    p : float
+        The p-value of the test.
+    m : float
+        The grand median.
+    table : ndarray
+        The contingency table.  The shape of the table is (2, n), where
+        n is the number of samples.  The first row holds the counts of the
+        values above the grand median, and the second row holds the counts
+        of the values below the grand median.  The table allows further
+        analysis with, for example, `scipy.stats.chi2_contingency`, or with
+        `scipy.stats.fisher_exact` if there are two samples, without having
+        to recompute the table.
+
+    See Also
+    --------
+    kruskal : Compute the Kruskal-Wallis H-test for independent samples.
+    mannwhitneyu : Computes the Mann-Whitney rank test on samples x and y.
+
+    Notes
+    -----
+    .. versionadded:: 0.15.0
+
+    References
+    ----------
+    .. [1] Mood, A. M., Introduction to the Theory of Statistics. McGraw-Hill
+        (1950), pp. 394-399. 
+    .. [2] Zar, J. H., Biostatistical Analysis, 5th ed. Prentice Hall (2010).
+        See Sections 8.12 and 10.15.
+
+    Examples
+    --------
+    A biologist runs an experiment in which there are three groups of plants.
+    Group 1 has 16 plants, group 2 has 15 plants, and group 3 has 17 plants.
+    Each plant produces a number of seeds.  The seed counts for each group
+    are::
+
+        Group 1: 10 14 14 18 20 22 24 25 31 31 32 39 43 43 48 49
+        Group 2: 28 30 31 33 34 35 36 40 44 55 57 61 91 92 99
+        Group 3:  0  3  9 22 23 25 25 33 34 34 40 45 46 48 62 67 84
+
+    The following code applies Mood's median test to these samples.
+
+    >>> g1 = [10, 14, 14, 18, 20, 22, 24, 25, 31, 31, 32, 39, 43, 43, 48, 49]
+    >>> g2 = [28, 30, 31, 33, 34, 35, 36, 40, 44, 55, 57, 61, 91, 92, 99]
+    >>> g3 = [0, 3, 9, 22, 23, 25, 25, 33, 34, 34, 40, 45, 46, 48, 62, 67, 84]
+    >>> stat, p, med, tbl = median_test(g1, g2, g3)
+
+    The median is
+
+    >>> med
+    34.0
+
+    and the contingency table is
+
+    >>> tbl
+    array([[ 5, 10,  7],
+           [11,  5, 10]])
+
+    `p` is too large to conclude that the medians are not the same:
+    
+    >>> p
+    0.12609082774093244
+
+    The "G-test" can be performed by passing ``lambda_="log-likelihood"`` to
+    `median_test`.
+
+    >>> g, p, med, tbl = median_test(g1, g2, g3, lambda_="log-likelihood")
+    >>> p
+    0.12224779737117837
+
+    The median occurs several times in the data, so we'll get a different
+    result if, for example, ``ties="above"`` is used:
+
+    >>> stat, p, med, tbl = median_test(g1, g2, g3, ties="above")
+    >>> p
+    0.063873276069553273
+
+    >>> tbl
+    array([[ 5, 11,  9],
+           [11,  4,  8]]) 
+
+    This example demonstrates that if the data set is not large and there
+    are values equal to the median, the p-value can be sensitive to the
+    choice of `ties`.
+
+    """
+    ties = kwds.pop('ties', 'below')
+    correction = kwds.pop('correction', True)
+    lambda_ = kwds.pop('lambda_', None)
+
+    if len(kwds) > 0:
+        bad_kwd = kwds.keys()[0]
+        raise TypeError("median_test() got an unexpected keyword "
+                        "argument %r" % bad_kwd)
+
+    if len(args) < 2:
+        raise ValueError('median_test requires two or more samples.')
+
+    ties_options = ['below', 'above', 'ignore']
+    if ties not in ties_options:
+        raise ValueError("invalid 'ties' option '%s'; 'ties' must be one "
+                         "of: %s" % (ties, str(ties_options)[1:-1]))
+
+    data = [np.asarray(arg) for arg in args]
+
+    # Validate the sizes and shapes of the arguments.
+    for k, d in enumerate(data):
+        if d.size == 0:
+            raise ValueError("Sample %d is empty. All samples must "
+                             "contain at least one value." % (k + 1))
+        if d.ndim != 1:
+            raise ValueError("Sample %d has %d dimensions.  All "
+                             "samples must be one-dimensional sequences." %
+                             (k + 1, d.ndim))
+
+    grand_median = np.median(np.concatenate(data))
+
+    # Create the contingency table.
+    table = np.zeros((2, len(data)), dtype=np.int64)
+    for k, sample in enumerate(data):
+        nabove = count_nonzero(sample > grand_median)
+        nbelow = count_nonzero(sample < grand_median)
+        nequal = sample.size - (nabove + nbelow)
+        table[0, k] += nabove
+        table[1, k] += nbelow
+        if ties == "below":
+            table[1, k] += nequal
+        elif ties == "above":
+            table[0, k] += nequal
+
+    # Check that no row or column of the table is all zero.
+    # Such a table can not be given to chi2_contingency, because it would have
+    # a zero in the table of expected frequencies.
+    rowsums = table.sum(axis=1)
+    if rowsums[0] == 0:
+        raise ValueError("All values are below the grand median (%r)." %
+                         grand_median)
+    if rowsums[1] == 0:
+        raise ValueError("All values are above the grand median (%r)." %
+                         grand_median)
+    if ties == "ignore":
+        # We already checked that each sample has at least one value, but it
+        # is possible that all those values equal the grand median.  If `ties`
+        # is "ignore", that would result in a column of zeros in `table`.  We
+        # check for that case here.
+        zero_cols = np.where((table == 0).all(axis=0))[0]
+        if len(zero_cols) > 0:
+            msg = ("All values in sample %d are equal to the grand "
+                   "median (%r), so they are ignored, resulting in an "
+                   "empty sample." % (zero_cols[0] + 1, grand_median))
+            raise ValueError(msg)
+
+    stat, p, dof, expected = chi2_contingency(table, lambda_=lambda_,
+                                              correction=correction)
+    return stat, p, grand_median, table
+
+
 def _hermnorm(N):
     # return the negatively normalized hermite polynomials up to order N-1
     #  (inclusive)
@@ -2000,7 +2211,7 @@ def _hermnorm(N):
     return plist
 
 
-def pdf_fromgamma(g1,g2,g3=0.0,g4=None):
+def pdf_fromgamma(g1, g2, g3=0.0, g4=None):
     if g4 is None:
         g4 = 3*g2*g2
     sigsq = 1.0/g2
